%
%			Knuth Bendix Completion Procedure
%
%   with iDb as metainterpreter to allow a interface to deal with
%   reduction steps and critical pair heuristics

:- module(mi_compl,[compProc_problem/4,
	compProc/4,
	startKB/3,
	startKB_problem/3]).

:- assertz(user:file_search_path(basic,'.')).
:- assertz(user:file_search_path(trs,'.')).

:- op(640,xfx,=>).
:- op(650,xfx,>-).
:- op(660,xfy,&).         % as =>, 
:- op(670,yfx,==>).        
:- op(670,xfy,<=).      
  
% unification operator, defined in mi_unif.pl
%:- op(650,xfx,<-).

:- use_module(library(lists),[remove_duplicates/2]).

:- use_module(basic(general)).                  % general predicates
:- use_module(basic(term)).
:- use_module(basic(queue)).

:- use_module(trs(mi_var)).                   % definitions of datastructures
:- use_module(trs(mi_rep)).
:- use_module(trs(mi_datastr)).               

:- use_module(basic(order)).                    % orderings
:- use_module(trs(mi_ford)).
:- use_module(trs(mi_size)).
:- use_module(trs(mi_emb)).
:- use_module(trs(mi_lebg)).
:- use_module(trs(mi_rord)).
:- use_module(basic(depthOrd)).                 % depthOrd

:- use_module(trs(mi_sort)).                  % and predicates to handle the databases
:- use_module(trs(mi_pdb)).

:- use_module(trs(mi_unif)).                  % unification, critical pairs,
:- use_module(trs(mi_cps)).                   % rule order.

:- use_module(trs(mi_idj)).                   % completion procedure

:- ensure_loaded(trs(mi_data)).                  

% startKBCompProc(+Name, +RedOrd, -Rules) <--
%         computes one step in the completion algortithm.
%         Rules is the list of rewrite rules generated in one step
%         State is the trs_state of the completion algorithm

compProc_problem(Name,RedOrd,Rules,State) :-
    equationAxioms(Name,Equations),
    is_equations(Equations),
    remove_duplicates(Equations,Eq1s),
    is_order(RedOrd),
    make_trs(RedOrd,Eq1s,State0),
    kbCompletionAlg(State0,State,Rules).

compProc(Equations,RedOrd,Rules,State) :-
    is_equations(Equations),
    remove_duplicates(Equations,Eq1s),
    is_order(RedOrd),
    make_trs(RedOrd,Eq1s,State0),
    kbCompletionAlg(State0,State,Rules).

kbCompletionAlg(trs(RedOrder,E1,R1,O1,CTree),trs(RedOrder,E,R,O,CTree),Rules) :-
    next_trs(trs(RedOrder,E1,R1,O1,CTree),trs(RedOrder,E,R,O,CTree)),
    queueHeads(R,Rules,queue(0,Ys,Ys)).
kbCompletionAlg(State0,State,Rules) :-
    next_trs(State0,State1),
    kbCompletionAlg(State1,State,Rules).

% kbCompletion(+Name, +RedOrd, -Rules) 
%       iterates the completion procedure till Rules represent a
%       complete rewrite system for the
%       list of equations corresponding to Name

startKB_problem(Name, RedOrd, Rules) :-
    equationAxioms(Name, Equations),
    is_equations(Equations),
    remove_duplicates(Equations,Eq1s),
    is_order(RedOrd),
    make_trs(RedOrd, Eq1s, State0),
    kbCompletion2(State0, _State, Rules).

startKB(Equations, RedOrd, Rules) :-
    is_equations(Equations),
    remove_duplicates(Equations,Eq1s),
    is_order(RedOrd),
    make_trs(RedOrd, Eq1s, State0),
    kbCompletion2(State0, _State, Rules).
 
%kbCompletion2(State0, Rules) :-
%    setof(R,State^kbCompletionAlg(State0,State,R),Rules).

kbCompletion2(State0,State,Rules) :-
    next_trs(State0,State1),
    !,
    kbCompletion2(State1,State,Rules).
kbCompletion2(trs(RedOrder,queue(0,Ys,Ys),R,O,CTree),trs(RedOrder,queue(0,Ys,Ys),R,O,CTree),Rules) :-  
    queueHeads(R,Rules,queue(0,Ys,Ys)).


make_trs(RedOrder,Es,trs(RedOrder,EQueue,queue(0,Ys,Ys),queue(0,Zs,Zs),CTree)) :-
    queueLasts_number(queue(0,Xs,Xs),Es,EQueue,Nrs),
    mapListTree(Nrs,CTree).

next_trs(trs(RedOrder,E1,R1,O1,CTree),trs(RedOrder,E,R,O,CTree)) :-
    iDJ_next(RedOrder,E1==>E,R1==>R,O1==>O,CTree),
    !.


test1_problem(Problem,Es,Os,Rs) :-
    compProc_problem(Problem,sizeOrd,Rs,trs(_,Es,_,Os,_)).

test2_problem(Problem,Rules) :-
    startKB_problem(Problem, sizeOrd, Rules).

start(Equations,Rules) :-
    startKB(Equations,sizeOrd,Rules).

:- compProc_problem(reduced_group,sizeOrd,Rules,_).

:- S = trs(sizeOrd,queue(3,[(u*1=i(i(u)))-1,(u*1=i(i(i(i(u)))))-2,(v*u=i(i(i(i(v))))*u)-3|_A],_A),queue(12,[(x*1=>x)-1,(i(x)*(x*y)=>y)-2,(i(u)*u=>1)-3,(i(1)=>1)-4,(1*(1*u)=>u)-5,(1*1=>1)-6,(i(i(v))*u=>v*u)-7,(u*i(u)=>1)-8,(v*(i(v)*u)=>u)-9,(u*i(i(i(u)))=>1)-10,(v*(i(i(i(v)))*u)=>u)-11,(u*i(i(i(i(i(u)))))=>1)-12|_B],_B),queue(19,[cp(i(u)*u,1)-1,cp(1,i(1))-2,cp(1*(1*u),u)-3,cp(1*1,1)-4,cp(i(1)*u,1*u)-5,cp(i(1)*1,1)-6,cp(i(i(v))*u,v*u)-7,cp(u*1,i(i(u)))-8,cp(u*i(u),1)-9,cp(1*1,i(1))-10,cp(u,v*(i(v)*u))-11,cp(1,u*i(i(i(u))))-12,cp(1*1,i(i(i(1))))-13,cp(u,v*(i(i(i(v)))*u))-14,cp(1*u,i(i(i(1)))*u)-15,cp(1,u*i(i(i(i(i(u))))))-16,cp(1*1,i(i(i(i(i(1))))))-17,cp(u*1,i(i(i(i(u)))))-18,cp(v*u,i(i(i(i(v))))*u)-19|_C],_C),node(1,[3,7|_D],_E,node(2,[1,4,5,6,2|_F],_G,node(5,[1,8,10,11,12|_H],node(3,[7|_I],_J,_K),node(7,[2,8,9,10|_L],_M,node(9,[10,11|_N],_O,_P)))))), is_trs_state(S).

%:- start([c=c_4,f(c_4)=c_4,f(f(a))=c_4,f(f(b))=c_4,f(g(f(a)))=c_4,f(g(f(b)))=c_4,g(a)=c_4,g(b)=c_4,g(c_4)=c_4,g(g(f(a)))=c_4,g(g(f(b)))=c_4],R).








