% mi_cps.pl
%
% rewrite and critical-pair predicates

:- module(mi_cps,[rewrite/4,
	cp/3]).

:- assertz(user:file_search_path(basic,'.')).
:- assertz(user:file_search_path(trs,'.')).

:- use_module(basic(term)).
:- use_module(trs(mi_var)).
:- use_module(trs(mi_unif)).	

% Definiton:
% a Position in a list of numbers selecting a special subterm in a term
% 
% Definition: 
% a term S rewrites to T, if a subterm S' of S at postion p match with the
% left-hand side of a rule (L => R) 
% formal: T = sigma(S[p <-- R])

% rewrite(S,RewRule,Pos,T) <-- 
%	S rewrites to T via RewRule at Pos
%           	S and T are ground terms with special user-variables

rewrite(S,RewRule,Pos,T) :-
	ground_to_nonground(RewRule,L_ng => R_ng),
	subterm(S,Pos,Sub_S),	 
	ground_to_nonground(Sub_S,Sub_S_ng,[],Di),
	match(Sub_S_ng,L_ng),
	nonground_to_ground(R_ng,R,Di,Di),
	substitute_pos(S,Pos,R,T). 

:- \+ rewrite(x*y,x*1=>x,Pos,T).

% Definition :
%
% let L => R and S => T be two rules of the database; 
% the equation sigma(T) = sigma(S[p <-- R]) 
% sigma a substitution, p a position
% is a critical pair, iff there is a omega(S), omega a substitution, 
% so that omega(S) rewrite to omega(sigma(T)) and
% omega(sigma(S[p <-- R])) [Dershowisz & Jounand p. 82]
% or
% if there exists a nonvariable subterm S' of S at positon p which is 
% unifiable with L, so that sigma(S') and S have no variables in common,
% then form sigma(S[p <-- R]) and sigma(T) a critical pair [Winkler]
%

%
% BUG cp funktioniert derzeit nur f"ur NICHT user-vars !!!!!!!!!!!1
%

% cp(TopRule,RewRule,cp(Cp_1,Cp_2),) <--
%	Cp_1 & Cp_2 is a critical pair as 
%	defined above, formed by TopRule & RewRule

cp(TopRule,RewRule,cp(Cp_1,Cp_2)) :-
        \+ contains_list([TopRule,RewRule]),
	ground_to_nonground(TopRule,TopRule_ng),
	ground_to_nonground(RewRule,RewRule_ng),
	cp_non_ground(TopRule_ng,RewRule_ng,Cp_1,Cp_2).
cp(TopRule,RewRule,CriticalPair) :-
        contains_list([TopRule,RewRule]),
	ground_to_nonground(TopRule,TopRule_ng),
	ground_to_nonground(RewRule,RewRule_ng),
	cp_non_ground(TopRule_ng,RewRule_ng,Cp_1,Cp_2),
	nonground_to_ground(cp(Cp_1,Cp_2),CriticalPair).

cp_non_ground((S => T),(L => R),Cp_1,Cp_2) :-
	cpStructure((L => R),Pos,(S => T)), 
	substitute_pos(S,Pos,R,Cp_1),		  
	Cp_2=T,
	\+ Cp_1==Cp_2.

cp_non_ground_2((S => T),(L => R),Cp_1,Cp_2) :-
	cpStructure((S => T),Pos,(L => R)), 
	substitute_pos(L,Pos,T,Cp_1),		  
	Cp_2=R,
	\+ Cp_1==Cp_2.

% cpStructure(RewRule,Pos,TopRule) <-- 
%	Rewrite_rule overlaps TopRule at Pos
%	and thus form a critical pair
%	the notion of such cp structures it due to
%	Winkler 

cpStructure((L => _R),Pos,(S => _T)) :-
	subterm(S,Pos,Sub_S),		 
	nonvar(Sub_S),			 
	unify(Sub_S,L).			

:- cpStructure((i(X)*X => 1),Pos,(U*1 => U)), Pos= [].
:- cpStructure(1*U=>U,Pos,i(X)*(X*Y)=>Y),substitute_pos(i(X)*(X*Y),Pos,U,T).
:- substitute_pos(i(X)*(X*Y),[2],U,T).
:- subterm(i(X)*(X*Y),Pos,Sub), nonvar(Sub), unify(Sub,1*U).
:- unify(X*Y,1*U).
:- cp_non_ground(i(X)*(X*Y)=>Y,1*U=>U,Cp1,Cp2).
:- ground_to_nonground(i(i(x))=>x,TopRule), ground_to_nonground(i(x)*(x*y)=>y,RewRule),cp_non_ground(RewRule,TopRule,Cp1,Cp2).
