
% mi_emb.pl
%
% embedding ordering

:- module(mi_emb,[embedding/2,
	strictEmbedding/2]).

:- assertz(user:file_search_path(basic,'.')).
:- assertz(user:file_search_path(trs,'.')).

:- use_module(trs(mi_var)).
%
% Definition: homeomorphic embedding relation ( |>= )
%
% let S = f(X1, X2, ..., Xm) and T = g(Y1, Y2, ..., Ym)
% S |>= T  iff
%
%   (i)   S = T
%   (ii)  Xi |>= T   for some i, 1 <= i <= m  
%   (iii) f = g and
%	 Xij |>= Yj for all j, 1 <= j <= n
%				 1 <= i1 < i2 ... < in <= m

embedding(S,S) :-		  % the terms were decomposed
	is_user_var(S),     % so check the identity at the bottom
	!.
embedding(S,T) :-		  % Xi |>= Y
	nonvar(S),
	S =.. [_F|Arg_Ss],
	embedding_some(Arg_Ss,T),
	!.
embedding(S,T) :-		  % Xij |>= Yj
	nonvar(T), 
	S =.. [F|Arg_Ss],
	T =.. [F|Arg_Ts],
	embedding_list(Arg_Ss,Arg_Ts).

embedding_some([S|_Arg_Ss],T) :-
	embedding(S,T).
embedding_some([_S|Ss],T) :-
	embedding_some(Ss,T).

embedding_list(_Ss,[]).
embedding_list([S|Ss],[T|Ts]) :-
	embedding(S,T),
	embedding_list(Ss,Ts).
embedding_list([_S|Ss],Ts) :-
	embedding_list(Ss,Ts).

%  strictEmbedding(S,T) <-- S |> T 

strictEmbedding(S,T) :-
	\+ S=T,
	embedding(S,T).

:- strictEmbedding((((--(----(0+1))+1)+1)+1)+1,--(0+1)).

