% mi_lebg.pl
% 	predicates to handles lexicograhich and bag orderings

:- module(mi_lebg, [lexOrd/3,
	strictLexOrd/3,
	bagOrd/3,
	strictBagOrd/3]).

:- assertz(user:file_search_path(basic,'.')).
:- assertz(user:file_search_path(trs,'.')).

:- use_module(library(lists),[select/3]).

:- use_module(basic(term)).
:- use_module(basic(order)).

% Definition: lexicographic ordering ( >=lex ) 
%
% let X = (X1, X2, ..., Xm) and Y = (Y1, Y2, ... , Yn)
% > represents the underlying term ordering
% X >=lex Y iff
%
%   (i) X1 > Y1
%   (ii) X1 = Y1 and X \ {X1} >=lex Y \ {Y1}

lexOrd(_Order,_S,[]). 
lexOrd(Order,[S|_],[T|_]) :-   
	strict_order(Order,S,T).
lexOrd(Order,[S|Ss],[T|Ts]) :- 
	equal(Order,S,T),
	lexOrd(Order,Ss,Ts).

strictLexOrd(Order,S,T) :-
	\+ S=T,
	lexOrd(Order,S,T).

%:- lexOrd(natOrd,[3,3,4,0],[3,3,3]).

%
% Definition: multiset ordering ( >>= )
%
% let X = [X1, X2, ..., Xm] and Y = [Y1, Y2, ... , Yn]
% >= represent the underlying term ordering
%
% X >>= Y iff
%
% (i)  Xi = Yi  and
%	  X - {Xi} >>= Y - {Yi}  for some i, 1 <= i <= m	
% (ii) Xi > Yj1, Yj2, ... Yjk and
%	  X - {Xi} >>= Y - [Yj1, Yj2, ... Yjk],
%	  for some i, 1 <= i <= m  and 1 <= j1 < ... < n (k >= 0)

bagOrd(_Order,_S,[]).	     % the second bag is empty
bagOrd(Order,[S|Ss],[T|Ts]) :-  % Def (i)
	equal(Order,S,T),
	bagOrd(Order,Ss,Ts).
bagOrd(Order,Ss,Ts) :-	        % Def (ii)
	select(S,Ss,S1s),	   
	partition(Order,Ts,S,_Ls,Hs),
	bagOrd(Order,S1s,Hs).

strictBagOrd(Order,S,T) :-	  % at least one element in S has
	\+ perm_congruence(S,T),	  % no coresponding element in T
	bagOrd(Order,S,T).

:- strictBagOrd(natOrd,[3,3,4,0],[3,2,2,1,1,1,4,0]).
