% mi_rep.pl
% representation for the metainterpretical completion procedure

:- module(mi_rep,[is_equation/1,
	is_rule/1,
	is_critical_pair/1,
	formel/3,
	is_fsymbol_weight/1,
	is_order/1]).

:- (Step&Step1&Step2 ==> C) = (A&B&D=>E >- F=>G ==> C).

is_equation((_L = _R)).

is_rule((_L => _R)).

is_critical_pair(cp(_Cp_1,_Cp_2)).

formel((S=T),S,T).
formel((S=>T),S,T).
formel(cp(S,T),S,T).

% is_fsymbol_weight(FsW) <--
%	specifies the syntax to represent the weight
%	function symbols 

is_fsymbol_weight((Fs = Weight)) :-
	nonvar(Fs),
	integer(Weight).

% is_fsymbol_weights(Fsws) <--
%	Fsws is a list of weights of function symbols

is_fsymbol_weights([FsW]) :-
	is_fsymbol_weight(FsW).
is_fsymbol_weights([FsW|FsWs]) :-
	is_fsymbol_weight(FsW),
	is_fsymbol_weights(FsWs).

% is_order(RedOrd) <--
% 	defines all possible reduction orderings

is_order(sizeOrd).
is_order(natOrd).
is_order(sizeOrdWeight(Ws)) :-
	is_fsymbol_weights(Ws).
is_order(rpo(Ws)) :-
	is_fsymbol_weights(Ws).
is_order(lpo(Ws)) :-
	is_fsymbol_weights(Ws).
is_order(kbo(Order)) :-
	is_order(Order).
is_order(bag(Order)) :-
	is_order(Order).
is_order(lex(Order)) :-
	is_order(Order).
is_order(depthOrd).

