
% mi_rord.pl
%
% rule ordering 

:- module(mi_rord,[encompassmentOrder/2,
	strictEncompassmentOrder/2,
	ruleOrd/3]).

:- assertz(user:file_search_path(basic,'.')).
:- assertz(user:file_search_path(trs,'.')).

:- use_module(basic(term)).
:- use_module(basic(order)).
:- use_module(trs(mi_pdb)).
:- use_module(trs(mi_unif)).

% Definition: encompassment orderering ( >=enc ) 
% 
% S >enc T if
% 
%   Sub_S is a subterm of S and
%   Sub_S is an instance of T

% encompassmentOrder(S,T) <--
% 	S >=enc T

encompassmentOrder(S,T) :-
	subterm(S,_Pos,Sub_S),
	ground_to_nonground(T,T_ng),
	match(Sub_S,T_ng).

% strictEncompassmentOrder(S,T) <--
% 	S >enc T

strictEncompassmentOrder(S,T) :-       % a proper subterm is requested
	subterm(S,_Pos,Sub_S),
        \+ S==Sub_S,
	ground_to_nonground(T,T_ng),
	match(Sub_S,T_ng).

% Definition: rule ordering in [De90] ( >rule )
% 
% (S => T) >rule (L => R) if
% 
%  (i) S >enc L
%  (ii) S =ren L and T >redord R

% ruleOrd(RedOrd, Rule1, Rule2) <--
% 	S >rule T

ruleOrd(_RedOrd, (S => _T), (L => _R)) :-
	strictEncompassmentOrder(S,L).
ruleOrd(RedOrd, (S => T), (L => R)) :-
	renaming_sv(S,L),
	redOrd(RedOrd,T,R).

:- ruleOrd(sizeOrd,i(x)*(x*y) => y,x*y => a).
:- ruleOrd(sizeOrd,i(u)*(u*v) => v*1,i(x)*(x*y) => y).
:- \+ ruleOrd(sizeOrd,i(a)*(a*b) => 1,i(x)*(x*y) => y).
:- \+ ruleOrd(sizeOrd,i(A)*(A*B) => 1,i(X)*(X*Y) => Y).
:- strictEncompassmentOrder(i(a*b)*c,x*y).
:- ruleOrd(sizeOrd,i(X)*(X*Y) => Y,X*Y => A).
