
% mi_sort.pl
%
% some usual sorting predicates like quicksort with second-order features
% to accomodate the underlying ordering

:- module(mi_sort,[quicksortAxiomLhs/3,
	partitionAxiomLhs/3]).

:- assertz(user:file_search_path(basic,'.')).
:- assertz(user:file_search_path(trs,'.')).

:- use_module(basic(queue)).
:- use_module(basic(order)).
:- use_module(trs(mi_lebg)).
:- use_module(trs(mi_rep)).

% quicksortAxiomLhs(Order,Xs,Ys) <--
%	Ys is a sorted queue of axioms Xs
%	sorted by a quicksort-algorithmen via Order

quicksortAxiomLhs(_Order, queue(0,Xs,Xs), queue(0,Xs,Xs)).
quicksortAxiomLhs(Order, XQueue0, YQueue) :-
	queueLength(XQueue0,Length),
	Length > 0,
	queueHead(XQueue0,X-Index,XQueue),
	partitionAxiomLhs(Order,XQueue,X,LQueue,HQueue),
	quicksortAxiomLhs(Order,LQueue,YQueue0),
	quicksortAxiomLhs(Order,HQueue,YQueue2),
	queueLast(YQueue0,X-Index,YQueue1),
	queueMerge(YQueue1,YQueue2,YQueue).

% partitionAxiomLhs(Order,XQueue,Y,L,H) <--
%	L is a sample
%	of elements in XQueue which are lower than
%	Y in the sense of Order
%	H is XQueue \ L

partitionAxiomLhs(_Order,queue(0,Xs,Xs),_Term,queue(0,Ys,Ys),queue(0,Zs,Zs)).
partitionAxiomLhs(Order,XQueue0,Term,LQueue,HQueue) :-
	queueLength(XQueue0,Length),
	Length > 0,
	queueHead(XQueue0,X-Index,XQueue),
	formel(X,L,_R),
	strict_order(Order,Term,L), 
	partitionAxiomLhs(Order,XQueue,Term,LQueue0,HQueue),
	queueLast(LQueue0,X-Index,LQueue).
partitionAxiomLhs(Order,XQueue0,Term,LQueue,HQueue) :-
	queueLength(XQueue0,Length),
	Length > 0,
	queueHead(XQueue0,X-Index,XQueue),
	formel(X,L,_R),
	order(Order,L,Term),
	partitionAxiomLhs(Order,XQueue,Term,LQueue,HQueue0),
	queueLast(HQueue0,X-Index,HQueue).

:- partitionAxiomLhs(sizeOrd,queue(1,[(1*x=x)-1|Xs],Xs),1*x,L,H).
:- partitionAxiomLhs(sizeOrd,queue(3,[(1*x=x)-1,(x*1=x)-2,(i(x)*(x*y)=y)-3|Xs],Xs),1*x,L,H).
:- quicksortAxiomLhs(sizeOrd,queue(3,[(x*1=x)-1,(i(x)*(x*y)=y)-2,(1*x=x)-3|Xs],Xs),Sort),Sort = queue(3,[(1*x=x)-3,(x*1=x)-1,(i(x)*(x*y)=y)-2|_A],_A). 
	
:- strictBagOrd(natOrd,[3,3,4,0],[3,2,2,1,1,1,4,0]).
:- lexOrd(natOrd,[3,3,4,0],[3,3,3]).
