
% mi_var.pl
%

:- module(mi_var,[user_var/1,
	next_user_var/1,
	is_user_var/1,
	non_user_var/1,
	contains_user_var/1,
	contains_list/1]).

% user defined 'variables'

user_var(u).
user_var(v).
user_var(w).
user_var(x).
user_var(y).
user_var(z). 

% next_user_var(X) <-- enumerates all user defined variables

next_user_var(X) :-
	user_var(X).

% is_user_var(X) <-- checks wether X is a user defined variable

is_user_var(X) :-
	nonvar(X),
	user_var(X).

% non_user_var(X) <-- same as nonvar but in respect of user defined variables

non_user_var(X) :-
	\+ is_user_var(X).

% contains_user_var(+T) <---
%                      checks wether T contains user variables

contains_user_var(U) :-
    nonvar(U),
    is_user_var(U).
contains_user_var(T) :-
    nonvar(T),
    T =.. [_F|TArgs],
    contains_list(TArgs).

contains_list([T|_Ts]) :-
    contains_user_var(T),
    !.
contains_list([_T|Ts]) :-
    contains_list(Ts).

:- contains_user_var(f(X,y)).
:- \+ contains_user_var(f(X,Y)).
