
% order.pl
%
% second-order predicates

:- module(order,[redOrd/3,
	order/3,
	strict_order/3,
	equal/3,
	maximum/3,
	minimum/3,
        quicksort/3,
	quicksort/4,
	partition/5,
	key_quicksort/3,
	key_quicksort/4,
	key_partition/5]).

:- assertz(user:file_search_path(basic,'.')).
:- assertz(user:file_search_path(trs,'.')).

:- use_module(basic(depthOrd)).
:- use_module(trs(mi_ford)).
:- use_module(trs(mi_lebg)).
:- use_module(trs(mi_emb)).
:- use_module(trs(mi_size)).	

% redOrd(Order,S,T) <-- 
%	is true iff S is greater than T via Order -
%	as an reduction order - and
%	under the list of function symbols weights
%	Weights

redOrd(Order,S,T) :-
	strict_order(Order,S,T).

% order(Order,S,T) <-- 
%	second-order predicate apply Order to S & T
%

% Remark:
% in all orders S = T means S is the same as T 
% I have not defined any equivalence classes
% renaming(S,T) means S is literaly similar to T; 

order(rpo(Ws),S,T) :-		  % greater or equal than
	rpo(Ws,S,T).
order(lpo(Ws),S,T) :-
	lpo(Ws,S,T).
order(kbo(Order,Ws),S,T) :-
	kbo(Order,Ws,S,T).
order(bagOrd(Order),S,T) :-
	bagOrd(Order,S,T).
order(lexOrd(Order),S,T) :-
	lexOrd(Order,S,T).
order(sizeOrd,S,T) :-
	sizeOrd(S,T).
order(sizeOrdWeight(Ws),S,T) :-
	sizeOrd_weight(Ws,S,T).
order(natOrd,S,T) :-
	S >= T.
order(embedding,S,T) :-
	embedding(S,T).
order(depthOrd,S,T) :-            % depthOrd/2 is defined in file depthOrd.pl
        depthOrdEq(S,T).
order(depth,S,T) :-               % depth/2 is defined in file depthOrd.pl
        depth(S,N),
        depth(T,M),
        N >= M.

strict_order(sizeOrd,S,T) :-	  % strict greater than
	strictSizeOrd(S,T).
strict_order(sizeOrdWeight(Ws),S,T) :- 
	strictSizeOrdWeight(Ws,S,T).
strict_order(natOrd,S,T) :-
	S > T.
strict_order(rpo(Ws),S,T) :-
	strictRpo(Ws,S,T).
strict_order(lpo(Ws),S,T) :-
	strictLpo(Ws,S,T).
strict_order(kbo(Order,Ws),S,T) :-
	strictKbo(Order,Ws,S,T).
strict_order(bagOrd(Order),S,T) :-
	strictBagOrd(Order,S,T).
strict_order(lexOrd(Order),S,T) :-
	strictLexOrd(Order,S,T).
strict_order(embedding,S,T) :-
	strictEmbedding(S,T).
%strict_order(indexOrd,S,T) :-
%	strict_indexOrd(S,T).
strict_order(depthOrd,S,T) :-     % depthOrd/2 is defined in file depthOrd.pl
        depthOrd(S,T).
strict_order(depth,S,T) :-        % depth/2 is defined in file depthOrd.pl
        depth(S,N),
        depth(T,M),
        N > M.

equal(natOrd,S,T) :-		  % equal
	!,
	S=:=T.
%equal(indexOrd,S,T) :-
%	!,
%	S=:=T.
equal(sizeOrd,S,T) :-
	!,
	size(S,Size_S),
	size(T,Size_T),
	Size_S=:=Size_T.
equal(rpo(_Ws),S,T) :-
	!,
	perm_congruence(S,T).
equal(bagOrd(_Order),S,T) :-
	!,
	perm_congruence(S,T).
equal(depth,S,T) :-
        !,
        depth(S,N),
        depth(T,M),
        N=:=M.
equal(_Order,S,T) :-
	S=T.


%    Max is the maximum in Xs under StrictOrder

maximum(StrictOrder, [X|Xs], Max) :-
	maximum(StrictOrder,Xs,X,Max).

maximum([], _StrictOrder, Max, Max).
maximum([X|Xs], StrictOrder, OldMax, Max) :-
	strict_order(StrictOrder,X,OldMax),
	!,
	maximum(Xs,StrictOrder,X,Max).
maximum([_X|Xs], StrictOrder, OldMax, Max) :-
	maximum(Xs,StrictOrder,OldMax,Max).

% minimum(+StrictOrder, +Xs, -Min) <--
%    Min is the minimum in Xs under StrictOrder

minimum(StrictOrder, [X|Xs], Min) :-
	minimum(Xs,StrictOrder,X,Min).

minimum([],_StrictOrder, Min, Min).
minimum([X|Xs], StrictOrder, OldMin, Min) :-
	strict_order(StrictOrder,OldMin,X),
	!,
	minimum(Xs,StrictOrder,X,Min).
minimum([_X|Xs], StrictOrder, OldMin, Min) :-
	minimum(Xs,StrictOrder,OldMin,Min).

% quicksort(+Order, +Xs, +Ys) <--
%	Ys is a sorted list Xs
%	sorted by a quicksort-algorithmen via Order

quicksort(Order, Xs, Ys) :-
	quicksort(Order,Xs,Ys,[]).

quicksort(_Order, [], Xs, Xs).
quicksort(Order, [X|Xs], Ys, Zs) :-
	partition(Order,Xs,X,Ls,Hs),
	quicksort(Order,Ls,Ys,[X|Ys0]),
	quicksort(Order,Hs,Ys0,Zs).

% partition(Order,Xs,Y,Ls,Hs) <--
%	Xs in a lower (Ls) & higher (Hs) list in respect to Y

partition(_Order, [], _Term, [], []).
partition(Order, [X|Xs], Term, Ls0, Hs) :-
	strict_order(Order,Term,X), 
	!,
	Ls0 = [X|Ls],
	partition(Order,Xs,Term,Ls,Hs).
partition(Order, [X|Xs], Term, Ls, [X|Hs]) :-
	partition(Order,Xs,Term,Ls,Hs).
%  order(Order,X,Term)

:- quicksort(natOrd,[3,2,1],[1,2,3]).

% key_quicksort(+Order, +Xs, +Ys) <--
% 	Xs is a list of VarKey-Pairs and Ys is sorted under Order
% 	in respect to the Keys

key_quicksort(Order, Xs, Ys) :-
	key_quicksort(Order,Xs,Ys,[]).

key_quicksort(_Order, [], Xs, Xs).
key_quicksort(Order, [X|Xs], Ys, Zs) :-
	key_partition(Order,Xs,X,Ls,Hs),
	key_quicksort(Order,Ls,Ys,[X|Ys0]),
	key_quicksort(Order,Hs,Ys0,Zs).

% key_partition(Order,Xs,Key-Value,Ls,Hs) <--
%	partition Xs in a lower (Ls) & higher (Hs) list in respect to Key

key_partition(_Order, [], _KeyValue, [], []).
key_partition(Order, [XKey-X|Xs], Key-Value, Ls0, Hs) :-
	strict_order(Order,Key,XKey), 
	!,
	Ls0 = [XKey-X|Ls],
	key_partition(Order,Xs,Key-Value,Ls,Hs).
key_partition(Order, [XKey-X|Xs], KeyValue, Ls, [XKey-X|Hs]) :-
	key_partition(Order,Xs,KeyValue,Ls,Hs).
%  order(Order,XKey,Key)

:- key_partition(natOrd,[1-[1,0,0],2-[0,1,0],3-[0,0,1]], 2-[0,1,0], Prop1,Prop). 
:- key_quicksort(natOrd,[1-5,5-3,3-2],Ys), Ys = [1-5,3-2,5-3]. 
:- key_quicksort(natOrd,[1-[1,0,0],2-[0,1,0],3-[0,0,1]], Prop).

% all_keys(KeyValues, Keys) <--
% 	Keys is a list of the keys in KeyValues

all_keys([], []).
all_keys([Index-_X|IndProps], [Index|Indices]) :-
	all_keys(IndProps,Indices).
