% queues.pl
% 

:- module(queue,[is_queue/1,
	queueLength/2,
	queueLast/3,
	queueLast_number/4,
	queueLast_Index/4,
	reverseQueue/2,
	queueHead/3,
	queueHeads/3,
	queueLasts/3,
	queueLasts_number/4,
	queueMerge/3,
	queueMember/2,
	queueSelect/3]).
	

% is_queue(queue(N,Xs,Ys)) <--
% 	defines a queue via a difference list Xs\Ys and a counter
% 	N, which represents the number of elements in the queue

is_queue(queue(0,Xs,Xs)).
is_queue(queue(N,[_|Xs],Ys)) :-
	integer(N),
	N > 0,
	N0 is N - 1,
	is_queue(queue(N0,Xs,Ys)).

:- is_queue(queue(3,[1*a=a,a*1=a,i(a)*(a*b)=b|Xs],Xs)).

% queueLength(+Queue, -Length) <--
% 	Length is the length of Queue

queueLength(queue(N,_,_),N).

% queueLast(+Queue0, +El, -Queue) <-- 
% 	adds El at the end of Queue0 

queueLast(queue(N0,Xs,[El|Ys]),El,queue(N,Xs,Ys)) :-
	N is N0 + 1.

:- queueLast(queue(0,Xs,Xs),x*1,Queue).

% queueLast_number(+Queue0, +El, -Queue, Number) <--
% 	adds Es at the end of Queue0 and index these elements according to
% 			elements in Queue0

queueLast_number(queue(0,Xs,[El-1|Ys]),El,queue(1,Xs,Ys),1).
queueLast_number(Queue,El,queue(N,Xs,Ys),Number) :-
	queueLength(Queue,N0),
	N0 > 0,
	reverseQueue(Queue,queue(N0,[_X-Number0|_],_)),
	Queue = queue(N0,Xs,[El-Number|Ys]),
	Number is Number0 + 1,
	N is N0 + 1.

% queueLast_Index(+Queue0, +El, +Index, -Queue) <--
% 	adds Es at the end of Queue0 and index this element according to
% 			Index

queueLast_Index(queue(N0,Xs,[El-Index|Ys]),El,Index,queue(N,Xs,Ys)) :-
        N is N0 + 1.

% reverseQueue(Xs,Ys) <--
% 	Ys is the reversed queue Xs

reverseQueue(queue(0,Xs,Xs),queue(0,Ys,Ys)).
reverseQueue(queue(N0,[X|Xs],Ys),queue(N,As,Bs)) :-
	N0 > 0,
	N1 is N0 - 1,
	reverseQueue(queue(N1,Xs,Ys),queue(N2,As,[X|Bs])),
	N is N2 + 1.

:- reverseQueue(queue(3,[a,b,c|Xs],Xs),Queue).
:- queueLast_number(queue(2,[a-1,c-3|Xs],Xs),d,Queue,Index).

% queueHead(+Queue0, -El, -Queue) <--
% 	El is the head of Queue0

queueHead(queue(N0,[El|Xs],Ys),El,queue(N,Xs,Ys)) :-
	N0 > 0,
	N is N0 - 1.

:- queueHead(queue(3,[1*x=x,x*1=x,i(x)*(x*y)=y|Xs],Xs),X,Queue).

% queueHeads(?Queue0, ?Ls, ?Queue) <--
% 	a list of elements are discarded from Queue0

queueHeads(Queue,[],Queue) :-
	is_queue(Queue).
queueHeads(Queue0, [X|Xs], Queue) :-
	queueHead(Queue0,X,Queue1),
	queueHeads(Queue1,Xs,Queue).

:- queueHeads(queue(4,[a,b,c,d|Xs],Xs),Ls,queue(1,[d|Xs],Xs)).

% queueLasts(+Queue0, +Ls, -Queue) <--
% 			a list of elements is added to Queue0

queueLasts(Queue,[],Queue) :-
	is_queue(Queue).
queueLasts(Queue0,[X|Xs],Queue) :-
	queueLast(Queue0,X,Queue1),
	queueLasts(Queue1,Xs,Queue).

% queueLasts_number(+Queue0, +Ls, -Queue, -Nrs) <--
%	adds Ls to Queue0 and index the elements in Ls
%	Nrs is a list of these indexes

queueLasts_number(Queue,[],Queue,[]) :-
	is_queue(Queue).
queueLasts_number(Queue0,[X|Xs],Queue,[N|Nrs]) :-
	queueLast_number(Queue0,X,Queue1,N),
	queueLasts_number(Queue1,Xs,Queue,Nrs).

:- queueLasts_number(queue(0,Xs,Xs),[a,b,c],Queue,Nrs).

% queueMerge(+Queue0, +Queue1, -Queue) <--
% 	append 2 Queues

queueMerge(queue(N0,Xs,Ys),queue(N1,Ys,Zs),queue(N,Xs,Zs)) :-
	N is N0 + N1.

:- queueMerge(queue(1,[1*x=x|Xs],Xs),queue(2,[x*1=x,i(x)*(x*y)=y|Ys],Ys),Merge).

% queueMemberXQueue) <--
% 	X is contained in XQueue

queueMember(X,queue(N,[X|_],_)) :-
	N > 0.
queueMember(X,queue(N,[_|Xs],Ys)) :-
	N > 1,
	N0 is N - 1,
	queueMember(X,queue(N0,Xs,Ys)).

:- queueMember(a,queue(3,[a,b,c|Xs],Xs)).
:- \+ queueMember(c,queue(2,[a,b|Xs],Xs)).
:- \+ queueMember(x,queue(3,[a,b,c|Xs],Xs)).

% queueSelect(X,XQueue) <--
% 	X is contained in XQueue

queueSelect(queue(N0,[X|Xs],Ys),X,queue(N,Xs,Ys)) :-
	N0 > 0,
	N is N0 -1.
queueSelect(queue(N0,[Z|Xs],Ys),X,Queue) :-
	N0 > 1,
	N is N0 - 1,
	queueSelect(queue(N,Xs,Ys),X,Queue0),
	queueMerge(queue(1,[Z|Zs],Zs),Queue0,Queue).

:- queueSelect(queue(3,[a,b,c|Xs],Xs),X,XQueue).

:- is_queue(Queue0),queueLast(Queue0,a,Queue),Queue=queue(1,[a|_],_).
