%%%%%%%%
%
% get_pos_clause <------
%
% Input: Spass_Output file, 
% Output: file ext. *.arm
%         consisting of a list of positive clauses (written as lists)
%         extracted from Spass_Output file
%         the literals are in TPTP syntax
%
% needs: member, select, append, read_file_as_list/2, parse_dfg/2
%

:- module(elimneg,[experMB/2]).

:- assertz(user:file_search_path(exper,'.')).

:- use_module(library(lists), [member/2]).
:- use_module(library(lists), [append/3]).
:- use_module(library(lists),   [select/3]).
:- use_module(exper(dfg_parser2), [ % <DFG>
        read_file_as_list/2,
	parse_dfg_tail/2,
	saturate_info/8
   ]).

:-op(500, fx, not).
:-op(500, xf,:).
:- op(500,fx,++).
%:- op(500,fx,--).

experMB(SPASS_Output_file,Arm_file) :-
	atom(SPASS_Output_file),
	atom(Arm_file),
	read_file_as_list(SPASS_Output_file,FileList),
	parse_dfg(FileList,problem(_,_,logical_part(_,_,_,[formulae(_,_,C)],_),_)),
	open(Arm_file,write,Stream),
	write('[printing '),
	write(Arm_file),
	write(']'),
	nl,
	check_clause(Stream,C).

check_clause(_S,[]).
check_clause(S,[_N:C|Cs]) :-
	pars_list3(C,C2),
	print(C2),
	print('.'),
        nl,
	write(S,C2),
	format(S,".~n",[]),
	check_clause(S,Cs),
	!.
check_clause(S,[_C|Cs]) :-
	check_clause(S,Cs).


pars_list3(C0,Cs) :-
    pars_list(C0,Cs0),
    !,
    pars_list4(Cs0,Cs).

pars_list4([],[]).
pars_list4([C|Cs], [++C|Cs]) :-
    \+ (C =.. [not|_]).


pars_list(C,C2) :-
	\+ var(C),
	C=.. [forall|[_A|Cs]],
	!,
	pars_list2(Cs,C2).
pars_list(C,C2) :-
	\+ var(C),
	C=.. [or|Cs],
	!,
	pars_list2(Cs,C2).
%pars_list(C,[C|C2]) :-
%	\+ var(C),
%	C=.. [equal|Cs],
%	!,
%	pars_list2(Cs,C2).
pars_list(C,[C]).


pars_list2([],[]).
pars_list2([C|Cs],C2) :-
	pars_list(C,Cs0),
	!,
	pars_list2(Cs,Cs1),
	append(Cs0,Cs1,C2).
    
%pars_list2([C|Cs],[C|C2]) :-
%    pars_list2(Cs,C2).




