:- module(basic_preds,[mi_var/1,
	bind_vars_to_constant/3,
	select_choice/2,
	is_instance_of/2,
	prepend_to_listlist/2,
	unifiable/2,
	unify/3,
	unify_from_first/2,
	all_positive_are_unit/1,
	list_to_set_ident/2,
	list_to_set_struct/2,
	varlist_of_termlist/2,
	display_arm/1,
	write_arm/2]).

:- assertz(user:file_search_path(finMod,'.')).

?- use_module(finMod(compatibility)).

% from compatibility.pl

=@=(X,Y) :- struct_identical([X],[Y]).

?- op(700,xfx,=@=).


% var(T) does not function correctly...
mi_var(T) :-
	functor(T, '$VAR', 1).



/******************************
	bind_vars_to_constant(+ListA, +Const, -ListB):

	ListA, ListB: Liste von Termem
	Const: zu substituierende Konstante

	ersetzt alle in den Termen von ListA vorkommeden
	Variablen durch das Konstantensymbol Const. Ergebnis
	in ListB.

*******************************/
bind_vars_to_constant([],_,[]).
bind_vars_to_constant([T|Ts], Const, [Const|Ts1]) :- 
	mi_var(T), bind_vars_to_constant(Ts,Const,Ts1).
bind_vars_to_constant([T|Ts],Const,[T1|Ts1]) :-
	T =.. [F|Args],
	bind_vars_to_constant(Args, Const, Res),
	T1 =.. [F|Res],
	bind_vars_to_constant(Ts,Const,Ts1), !.	

% dasselbe predicate fuer einen einzelnen Term
bind_vars_to_constant(T,Const,T1) :-
	\+ T = [_],
	\+ T = [],
	bind_vars_to_constant([T],Const,[T1]).


/******************************

	select_choice(-Choice, +Possibilities)

	Choice: Liste
	Possibilities: Liste von Liste.

	Auswahlfunktion,
	waehlt eine Liste (Choice), die aus jeder Liste von Poss.
	ein Element enthaelt.

	Bsp.: select_choice(X,[[1,2],[3,4,5]]). liefert nacheinander
		die Antworten:

			X = [1,3] ;
			X = [1,4] ;
			X = [1,5] ;
			X = [2,3] ;
			X = [2,4] ;
			X = [2,5] ;

*******************************/

select_choice([],[]).
select_choice([E|Es], [P|Ps]) :-
	member(E,P),	
	select_choice(Es,Ps).


/******************************
	is_instance_of(+TermA, +TermB):

	succeeds, wenn TermA eine instanz von TermB ist.

*******************************/

is_instance_of(A,B) :-
	bind_vars_to_constant(A,hollodaro_gugu,A1),
	unifiable(A1,B).	



prepend_to_listlist([],A,A).
prepend_to_listlist([E|Es], [L|Ls], [[E|L]|Rs]) :-
	prepend_to_listlist(Es,Ls,Rs).


/******************************
	unifiable(A,B):

	testet, ob A und B unifizierbar sind, ohne
	eine Substitution durchzufuehren
*******************************/
unifiable(A,B) :-
	copy_term(A,A1),
	copy_term(B,B1),
	A1 = B1.

unify(A,B,C) :-
	copy_term(A,A1),
	copy_term(B,B1),	
	A1 = B1, C = A1.

/******************************
	unify_from_first(A,B)

	A .. Liste von Termen
	B .. Term

	unifiziert das erste Element von A mit B
	alle dafuer notwendigen Substitutionen werden
	auch auf alle anderen Elemente aus A angewandt.

*******************************/
unify_from_first([T|Ts], Unifier) :-
	copy_term(Unifier, H),
	length(Ts,N),
	length(Htail,N),
	[T|Ts] = [H|Htail]. 



is_unit_clause([_L|[]]).

is_positive_clause([]).
is_positive_clause([L|Ls]) :-
	\+ L = -_,
	is_positive_clause(Ls).

is_positive_unit_clause(C) :-
	is_positive_clause(C),
	is_unit_clause(C).

/*************************
*
*  all_positive_are_unit(A)
*
*	ueberprueft, ob in der klauselmenge A jede positive Klausel
*   unit-klausel ist.
*
**************************/
all_positive_are_unit([]).
all_positive_are_unit([C|Cs]) :-
	(is_positive_clause(C) -> is_unit_clause(C) ; true),
	all_positive_are_unit(Cs).


/*************************
*
*  list_to_set_ident(A,B)
*
*
**************************/

has_twin_ident(Ls) :-
	meta_select(Ls,A,H),
	meta_select(H,B,_),
	A == B,!.

remove_twin_ident([],[]).
remove_twin_ident([A|As],As) :-
	meta_select(As,E,_),
	E == A, !. 
remove_twin_ident([A|As], [A|Bs]) :-
	\+ remove_twin_ident([A|As],As),
	remove_twin_ident(As,Bs).

list_to_set_ident(As,As) :-
	\+ has_twin_ident(As).
list_to_set_ident(As,Bs) :-
	has_twin_ident(As),
	remove_twin_ident(As,H),
	list_to_set_ident(H,Bs).

/*************************
*
*  list_to_set_struct(A,B)
*
*
**************************/

has_twin_struct(Ls) :-
	meta_select(Ls,A,H),
	meta_select(H,B,_),
	A =@= B,!.

remove_twin_struct([],[]).
remove_twin_struct([A|As],As) :-
	meta_select(As,E,_),
	E =@= A, !. 
remove_twin_struct([A|As], [A|Bs]) :-
	\+ remove_twin_struct([A|As],As),
	remove_twin_struct(As,Bs).

list_to_set_struct(As,As) :-
	\+ has_twin_struct(As).
list_to_set_struct(As,Bs) :-
	has_twin_struct(As),
	remove_twin_struct(As,H),
	list_to_set_struct(H,Bs).

/*************************
*
*  varlist_of_termlist/2
*
*
**************************/

varlist_of_termlist([],[]).
varlist_of_termlist([T|Ts],[T|Vars]) :-
	mi_var(T),
	varlist_of_termlist(Ts,Vars).	
varlist_of_termlist([T|Ts],Vars) :-
	T =.. [_F|Args],
	varlist_of_termlist(Args,H1),
	varlist_of_termlist(Ts,H2),
	append(H1,H2,Vars).


is_linear_term(T) :-
	varlist_of_termlist([T],V),
	\+ has_twin_ident(V).

is_list_of_linear_terms([]).
is_list_of_linear_terms([T|Ts]) :-	
	is_linear_term(T),
	is_list_of_linear_terms(Ts).

	

number_range(A,A,[A]).
number_range(A,B,[A|Rest]) :-
	A < B,
	A1 is A+1,
	number_range(A1,B,Rest).


%---------------------------
% output routines
%
% display_arm/1
% write_arm/2
%---------------------------
	
% output syntax *
display_arm([]).
display_arm([A|R]) :- write('['), 
	write(A),
	write(']'), 
	write('.'), 
	nl, 
	display_arm(R).

write_arm(_S,[]).
write_arm(S,[L|Ls]) :- 
	write(S,[L]), 
	format(S,".~n",[]), 
	write_arm(S,Ls).

