%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% finite model building
% using/adapting files of the siemens-project and routines of clause evaluation
% scope: linear ARMs and GEAs
%
% new version, some bugs elim.
% and different output routine
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
:- module(fin, [finite_model/6,normal_form_term/3]).

:- assertz(user:file_search_path(finMod,'.')).
:- assertz(user:file_search_path(cl,'../cleval')).
:- assertz(user:file_search_path(exper,'../exper')).

:- use_module(library(system)).
:- use_module(library(lists),[append/3,remove_duplicates/2]).
:- use_module(exper(dfg_parser2), [read_file_as_list/2]).

:- use_module(finMod(herbrand)).
%:- use_module(finMod(basic_preds)).
:- use_module(finMod(compatibility)).
:- use_module(finMod(frontend)).
?- use_module(cl(cl_eval),[narrowing/3, narrowTerm/3, compSize/2, new_constant/2]).
?- use_module(cl(tptp_parser)). 

% necessary to parse dfg files
:-op(500,xf,:).

% reading in files

/*
reading_arm(Afile, Arm0, Eqs0) :-
	read_file_as_list(Afile, X),
	mb_parser(X, S),
	% syntax of equation: s=t
	S = eval(Arm0, Eqs0, _),
	!.
reading_arm(_,[],[]).
*/
reading_arm(A, A0, E0) :-
	open(A, read, St),
	reading_arm1(St, A0, E0),
	close(St), !.
reading_arm(_,[],[]).
reading_arm1(S, As, Es) :-
	read(S, Clause),
	Clause \== end_of_file,
	!,
        numbervars(Clause,0,_),
	reading_arm2(Clause, A, E),
	reading_arm1(S, A0, E0),
	append(A, A0, As),
	append(E, E0, Es).
reading_arm1(_,[],[]).
reading_arm2([++equal(X,Y)],[],[X=Y]) :- !.
reading_arm2([++A],[A],[]).


reading_problem_file(cnf, PFile, Fsig, Psig, Deqs) :-
	read_file_as_list(PFile,Filelist),
	parse_dfg(Filelist,problem(_,_,logical_part(symbol_list(Fsig,Psig,_,_,_),_,_,Clauses,_),_)),
	extract_disequations_dfg(Clauses,Deqs).
reading_problem_file(tptp, PFile, Fsig, Psig,Deqs):-
	open(PFile, read, Tstream),
	reading_tptp(Tstream, Ls),
	parse_tptp(Ls, Fsig, Psig, Deqs),
	close(Tstream).

parse_tptp(ProblemLits, Fsig, Psig,Deqs) :-
	remove_top_functor(ProblemLits, ProblemLits0),
	find_all_predicates(ProblemLits0, Psig),
	remove_top_functor(ProblemLits0, Ts),
	signature(Ts, Fsig),
	extract_disequations_tptp(ProblemLits,Deqs).

reading_tptp(T, Sig) :-
	read(T, Clause),
	Clause \== end_of_file,
	!,
        numbervars(Clause,0,_),
        read_clause_tptp(Clause, Sig_clause),
	reading_tptp(T, Sig2),
	append(Sig_clause, Sig2, Sig).		
reading_tptp(_,[]).

read_clause_tptp(input_clause(_,_,[]), []).
read_clause_tptp(input_clause(_,_,[ A | R]), [A | Sig]) :-
	read_clause_tptp(input_clause(_,_,R), Sig).


reading_setting(Sfile, S) :-
	file_exists(Sfile),
	!,
	open(Sfile, read, Sstream),
	read(Sstream, S),
	close(Sstream).
reading_setting(_, []).
	


% use routines of clause evaluation

normal_form(As0, Rules, As_n) :-
	narrowing(As0, Rules, As_n), 
	!.
normal_form(As0, _, As0).

normal_form_term(T, Rules, S) :-
	narrowTerm(Rules, T, S),
	!.
normal_form_term(_, T, T).

transform_to_rules([], []).
transform_to_rules(Eq, Rules) :-
	%atoms_equations(Eq, Eq1),
	compSize(Eq, Rules).	


extract_disequations_tptp([--equal(X,Y) | R], [equal(X,Y) | Deq]) :-
	!,
	extract_disequations_tptp(R, Deq).
extract_disequations_tptp([_|R], Deq) :-
	extract_disequations_tptp(R, Deq).
extract_disequations_tptp([],[]).

extract_disequations_dfg([],[]).
extract_disequations_dfg([formulae(_,_,Cs)|Clauses],Deqs) :-
	ext_dis2(Cs,Deqs0),
	extract_disequations_dfg(Clauses,Deqs1),
	append(Deqs0,Deqs1,Deqs).

ext_dis2([],[]).
ext_dis2([_N:C|Cs],Deqs) :-
	\+ var(C),
	C=.. [forall|[_A|Args]],
	!,
	ext_dis3(Args,Deqs0),
	ext_dis2(Cs,Deqs1),
	append(Deqs0,Deqs1,Deqs).
ext_dis2([_N:C|Cs],Deqs) :-
	\+ var(C),
	C=.. [or|Args],
	!,
	ext_dis3(Args,Deqs0),
	ext_dis2(Cs,Deqs1),
	append(Deqs0,Deqs1,Deqs).
ext_dis2([_N:C|Cs],Deqs) :-
	ext_dis3([C],Deqs0),
	ext_dis2(Cs,Deqs1),
	append(Deqs0,Deqs1,Deqs).

ext_dis3([C],Deqs) :-
	\+ var(C),
	C=.. [forall|[_A|Args]],
	!,
	ext_dis3(Args,Deqs).
ext_dis3([C],Deqs) :-
	\+ var(C),
	C=.. [or|Args],
	!,
	ext_dis3(Args,Deqs).
ext_dis3([A|Args],[EqAtom|Deqs]) :-
	\+ var(A),
	A=.. [not|[EqAtom]],
	EqAtom=..[equal|_Cs],
	!,
	ext_dis3(Args,Deqs).
ext_dis3([_A|Args],Deqs) :-
	ext_dis3(Args,Deqs).
ext_dis3([],[]).


%:- ext_dis2([3:forall(_B,or(not equal(_B,0),'Lo'(skf1(_B),_B)))],D).

%---snip---snip------------------------------------------------------
% main

finite_model(Flag,Arm_file,Problem_file,Setting_file,Fin_file,NewArm_file) :-
	% undo renaming
	open(Fin_file,write,FinStream),
	write('[printing '), write(Fin_file), write(']'),
	nl,
	open(NewArm_file,write,ArmStream),
	write('[printing '), write(NewArm_file), write(']'),
	nl,
	finite_model_stream(Flag,FinStream,ArmStream,Arm_file,Problem_file,Setting_file),
	close(ArmStream),
	close(FinStream).

finite_model_stream(Flag,SFin,SArm,Arm_file,Problem_file,Setting_file) :-
	reading_arm(Arm_file, Lits0, Eqs0),

	% clause set
	% signature
	reading_problem_file(Flag, Problem_file, Fsig, Psig, Deqs),
	new_constant(Fsig,NewCon),

	% setting
	reading_setting(Setting_file, Setting),

	%remove_top_functor(Arm, Arm0),
	%split_arm(Arm0, Ats, Eqs),
	transform_to_rules(Eqs0, Rules),
	normal_form(Lits0, Rules, Arm1),
	normal_form(Eqs0, Rules, Eqs1),
	domain_of_arm(Arm1, NewCon, DArm1),
	domain_of_arm(Eqs1, NewCon, DEqs1),
	append(DArm1, DEqs1, Domain0),

	%remove_top_functor(Deqs, Deqs0),
	normal_form(Deqs, Rules, Deq1),
	domain_of_arm(Deq1, NewCon, Domain1),

	% domain
	collect_domain(Domain0, Domain1, NewCon, Domain2),
	remove_duplicates(Domain2, Domain),	

	% *** save data ***
	assert(user:model_arm(Arm1)),
	assert(user:model_domain(Domain)),
	assert(user:model_rules(Rules)),
	assert(user:model_setting(Setting)),
	assert(user:model_signature(Psig, Fsig)),
	orthoset_of_arm(Arm1, Oset),
	assert(user:model_orthset(Oset)),

	nl,
	write('employ:'), 
	nl,
	write(SFin,'employ:'),
	format(SFin,"~n",[]),

	write('Normalized Atoms: '), write(Arm1), 
	nl,
	write(SFin,'Normalized Atoms: '), write(SFin,Arm1), 
	format(SFin,"~n",[]),

	write('Rules: '), print(Rules), 
	nl,
	write(SFin,'Rules: '), write(SFin,Rules),
	format(SFin,".~n",[]),
	
	write('Setting: '), write(Setting), 
	nl,
	write(SFin,'Setting: '), write(SFin,Setting),
	format(SFin,".~n",[]),	

	write('Functions: '), write(Fsig), 
	nl,
	write(SFin,'Functions: '), write(SFin,Fsig),
	format(SFin,".~n~n",[]),

	% do it
	analyze(SFin,SArm,NewCon),
	clear.

	

	

	
	
	
