%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% finite model building and arm extraction
% (siemens)
%
% new version for tilt
% new output routines
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
:- module(frontend, [display_arm/1,
	find_all_predicates/2,
	analyze/3,
	clear/0]).

:- use_module(library(lists)).

/* fuer die settings */
?- op(500, fx, ++).
?- op(500, fx, --).
% fuer rules
?- op(640, xfx, =>).

:- assertz(user:file_search_path(finMod,'.')).
%:- assertz(user:file_search_path(cl,'../cleval')).

:- use_module(finMod(herbrand)).
:- use_module(finMod(basic_preds)).
:- use_module(finMod(compatibility)).
:- use_module(finMod(fin),[normal_form_term/3]).

% ****************

show_as_index([],_,[]).
show_as_index([E|Es], Set, [I|Is]) :-
	nth0(I,Set,E),
	show_as_index(Es,Set,Is).

functable(_, _, [], []).
functable(Fname/1, Dom, Dom, Table) :-
%        format("vor maplist ~w",[Dom]),nl,
	maplist(Fname,Dom,H),
%        format("nach maplist ~w",[H]),
	show_as_index(H,Dom,Table).
functable(Fprefix/N, Dom, [D|Ds], [T|Ts]) :-
	N > 1,
	Fprefix =.. [F|Pars],	
	append(Pars,[D],NewPars),
	FprefixNew =.. [F|NewPars],
	N1 is N-1,
	functable(FprefixNew/N1, Dom, Dom, T),
	functable(Fprefix/N, Dom, Ds, Ts).
	

/* make a table a a predicate */

predtable(_,_,[],[]).
predtable(Pnew/1, Dom, Dom, Table) :-
        maplist(Pnew,Dom,Table).
predtable(Pprefix/N, Dom, [D|Ds], [T|Ts]) :-
        N > 1,
        Pprefix =.. [P|Pars],
        append(Pars,[D],NewPars),
        PprefixNew =.. [P|NewPars],
        N1 is N-1,
        predtable(PprefixNew/N1, Dom, Dom, T),
        predtable(Pprefix/N, Dom, Ds, Ts).



pprint_domain(_SFin,[],_).
pprint_domain(SFin,[D|Ds], N) :-
	print(N), write(' = '),
	write(SFin,N),
	write(SFin,' = '),

	if(is_s0(D,DN),
	   (print('"'), print(DN), print('"'),
	   write(SFin,'"'), write(SFin,DN), write(SFin,'"') ),
	   (print(D),write(SFin,D))
	  ),
	nl,
	format(SFin,"~n",[]),
	N1 is N+1,
	pprint_domain(SFin,Ds,N1).

is_s0(0,0).
is_s0(s(X),N1) :-
	is_s0(X,N),
	N1 is N+1.

pprint_intlist([]).
pprint_intlist([L|Ls]) :-	
	writef('%4L', [L]),
	pprint_intlist(Ls).

pprint_array(N,Max,_) :- N > Max.
pprint_array(N,Max,[Line|Lines]) :-
	writef('%3L | ',[N]),
	pprint_intlist(Line), nl,
	N1 is N+1, 
	pprint_array(N1,Max,Lines). 

pprint_function(F/1,Table) :-
	writef('%3L | ',[F]),
	length(Table,N),
	N1 is N-1,
	number_range(0,N1,Range),
	pprint_intlist(Range), nl,
	print('-----------------------------------'), nl,
	writef('    | '),
	pprint_intlist(Table), 
	nl.

pprint_function(F/2, Table) :-
	writef('%3L | ',[F]),
	length(Table,N),
	N1 is N-1,
	number_range(0,N1,Range),
	pprint_intlist(Range), nl,
	print('-----------------------------------'), nl,
	pprint_array(0,N1,Table).	

pprint_function(_/N, _) :-
	N>2,
	print('cant display multidim function'),
	nl.

uglydisplay_function(F/N, Table) :-
	write('* function '), print(F/N),	
	nl,
	print(Table), 
	nl.

uglywrite_function(SFin,F/N, Table) :-
	write(SFin,'* function '), write(SFin,F/N),	
	format(SFin,"~n",[]),
	write(SFin,Table),
	format(SFin,"~n",[]).


display_functions(_SFin,[]).
display_functions(SFin,[F/Fnew/N|Fs]) :-
	user:model_domain(Dom),
%        write('Display: '),
%        write(Dom), nl,
	functable(Fnew/N,Dom,Dom,Table),
	uglydisplay_function(F/N,Table),
	uglywrite_function(SFin,F/N,Table),
	format(SFin,"~n",[]),
	display_functions(SFin,Fs).



%---snip---snip------------------------------------------------------
% prepare evaluation
	
assert_modelfunctions(_NC,[]).
assert_modelfunctions(NC,[_F/0|Fs]) :-
	assert_modelfunctions(NC,Fs).
assert_modelfunctions(NC,[F/N|Fs]) :-
	N > 0,
	name(F,Fname),
	append("model_",Fname,Fnewname),
	name(Fnew,Fnewname),
	length(Paramlist,N),	% liste von neuen variablen fuer parameter
	append(Paramlist,[Result], Arglist),
	H1 =.. [Fnew|Arglist],
	H2 =.. [F|Paramlist],
	assert(user:(H1 :- frontend:evaluate_fct(NC,H2,Result))),
	assert(user:modelfunction(F/Fnew/N)),  % funktion vermerken
%        write('Assert: '),
%        write(F), write(Fnew), write(N), nl,
        assert(user:functions_exist(yes)),
	assert_modelfunctions(NC,Fs).

assert_modelpredicates([]).
assert_modelpredicates([_P/0|Ps]) :-
	assert_modelpredicates(Ps).
assert_modelpredicates([P/N|Ps]) :-
	N > 0,
	name(P,Pname),
	append("modelp_",Pname,Pnewname),
	name(Pnew,Pnewname),
	length(Paramlist,N),	% liste von neuen variablen fuer parameter
	append(Paramlist,[Result], Arglist),
	H1 =.. [Pnew|Arglist],
	H2 =.. [P|Paramlist],
	assert(user:(H1 :- frontend:evaluate_pred(H2,Result))),
	assert(user:modelpredicate(P/Pnew/N)),  % funktion vermerken
%        write('Assert p :'),
%        write(P), write(Pnew), write(N),
	assert(user:predicates_exist(yes)),
	assert_modelpredicates(Ps).

%---snip---snip------------------------------------------------------


retract_modelfunctions([]).
retract_modelfunctions([_F/Fname/N|Fs]) :-
	N1 is N+1,
	abolish(user:Fname,N1),
	retract_modelfunctions(Fs).


retract_modelpredicates([]).
retract_modelpredicates([_F/Fname/N|Fs]) :-
        N1 is N+1,
        abolish(user:Fname,N1),
        retract_modelpredicates(Fs).

%----------------------------------------
% print_functions_if_present/1
%
%---------------------------------------

print_functions_if_present(SFin) :-
	user:functions_exist(yes),
	findall(X,user:modelfunction(X),AllFs),
	write('*** Functions:'), 
	nl,
	write(SFin,'*** Functions:'),
	format(SFin,"~n",[]),
        display_functions(SFin,AllFs),
	!.		 
print_functions_if_present(_SFin).



%---snip---snip------------------------------------------------------
% evaluations

/* evaluation von praedikaten */
% the same for linear arms and geas
/*
evaluate_pred(PredTerm, 1) :-
    user:model_arm(A),
    member(PredTerm,A),
	!.
evaluate_pred(_, 0).
*/

evaluate_pred(P, X) :-
	user:model_arm(A),
	% consider setting already here
	user:model_setting(S),
	P =.. [F|_],
	( member(P,A) ->
		( member(++F,S) -> X=0; X=1, assert(user:model_show(++P)) )
	;
		( member(++F,S) -> X=1, assert(user:model_show(++P)); X=0 )
	).	


/* evaluation von functionen */
% evaluation for linear ARMs

model_phi_pos(T,Orthset,NewCon,Res) :-
	member(X,Orthset),
	is_instance_of(T,X), 
	gamma_d(NewCon,[X],[Res]).

model_phi(T,O,NewCon,R) :-
	model_phi_pos(T,O,NewCon,H) ->  R = H; R = NewCon.


evaluate_fct(NewCon, Term, Result) :-
	user:model_rules(Rules),
	Rules == [],
	!,
	user:model_orthset(Os),
	model_phi(Term, Os, NewCon, Result),
	% the extracted arm has equations!
	( \+ (Term=Result) -> 
              assert(user:model_show(++equal(Term, Result)))
        ;
              true
        ).
%evaluation for GEAs
evaluate_fct(NewCon,Term, Result) :-
	user:model_rules(Rules),
	user:model_domain(Domain),
	normal_form_term(Term, Rules, Term_norm),
	( member(Term_norm, Domain) -> Result = Term_norm; Result = NewCon ),
	( \+ (Term=Result) -> 
              assert(user:model_show(++equal(Term, Result)))
        ;
              true
        ).



%---snip---snip------------------------------------------------------


/* alle praedikatsnamen + arity aus einer liste von termen suchen */ 
% equal/2 will be ignored

find_all_predicates1([],[]).
find_all_predicates1([X|Xs],[P|Ps]) :-
    functor(X,F,N),
    P = F/N,
    find_all_predicates1(Xs,Ps).

find_all_predicates(A,C) :-
	find_all_predicates1(A,H),
	list_to_set(H,B),
	remove_equal(B,C).

remove_equal(L, L1) :-
	select(equal/2, L, L1), !.
remove_equal(L, L).



/* invert table if predicate f appears among settings */
/* 
% changed member(--F,S) to member(++F,S)
% according to classifier
% GM
probably_invert(F/_,T,T1) :-
    user:model_setting(S),
    member(++F,S),
    invert_table(T,T1),
	!.

probably_invert(_/_,T,T).
    
invert_table([],[]).
invert_table(0,1).
invert_table(1,0).
invert_table([X|Xs],[Y|Ys]) :- 
    invert_table(X,Y),
    invert_table(Xs,Ys).
*/
    

% setting already considered in evaluation
% probably_invert(F/N,Tab,Tab1),
% print(Tab1), nl, nl,

tabulate_predicates1(_SFin,[]).
tabulate_predicates1(SFin,[P/PNew/N|Ps]) :-
	write('* predicate '), print(P), nl,
	format(SFin,"* predicate ~w ~n",[P]),
        user:model_domain(D),
	predtable(PNew/N,D,D,Tab),
	print(Tab), nl, nl,
	format(SFin,"~w ~n",[Tab]),
	tabulate_predicates1(SFin,Ps).

tabulate_predicates(SFin) :-
	user:predicates_exist(yes),
	findall(X,user:modelpredicate(X),All),
	tabulate_predicates1(SFin,All).
tabulate_predicates(_SFin) :-
	user:predicates_exist(not).



/* settings als liste einlesen und als model_predicate/1 speichern [old]
read_settings(Filename) :-
        see(Filename),
        repeat,
        read(X),
         (X == end_of_file ->
	       true
	  ;
	       assert(user:model_settings(X)),
	       fail
	  ),!,
        seen.
*/


/* main [old]
make_from_plfile(File) :-
	read_clauses_from_file(File,ClauseSet),
	arm(ClauseSet, H),
	flatten(H, Arm),
	write('*** ARM:'), nl, print(Arm), nl,nl,
        find_all_predicates(Arm,PredList),
        write('*** defined predicates: '), print(PredList), nl,
	analyze(Arm),
        assert_modelpredicates(PredList),
        tabulate_predicates.
*/

/* [old]
analyze(Arm) :-
	domain_of_arm(Arm,Dom),	
	orthoset_of_arm(Arm,Oset),
        assert(model_arm(Arm)),
	assert(model_domain(Dom)),
	assert(model_orthset(Oset)),
        assert(functions_exist(not)),
	signature(Oset,Sig),
	assert_modelfunctions(Sig),
	write('*** domain:'), nl,
	pprint_domain(Dom,0), nl,
	print_functions_if_present, 
        nl.
*/

/* alle dynamischen preaedikate entfernen [old]
clear :-
	abolish(model_domain,1),
	abolish(model_orthset,1),
        abolish(model_arm,1),
        abolish(functions_exist,1),
        findall(X1,modelpredicate(X1),All1),
        retract_modelpredicates(All1),
        abolish(modelpredicate,1),
	findall(X,modelfunction(X),All),
	retract_modelfunctions(All),
	abolish(modelfunction,1).
*/


constants_not_in_domain([], _, []).
constants_not_in_domain([F/0 | R], D, [F | R0]) :-
	\+ member(F, D),
	!,
	constants_not_in_domain(R, D, R0).
constants_not_in_domain([_/_| R], D, R0) :-
	constants_not_in_domain(R, D, R0).


% extract gea/arm from finite model
extract_arm(SArm,NewCon,Cs) :-
	get_new_literals(Ls),
	get_old_equations(Es1),
	get_eq_from_constants(Cs, NewCon, Es2),
	append(Es2, Es1, Es),
	append(Es, Ls, A),
	sort(A, Arm),
	write_arm(SArm,Arm).



get_new_literals(Ls) :-
	model_show_exists,
	findall(X, user:model_show(X), Ls),
	!.
get_new_literals([]).

model_show_exists :-
	user:model_signature(Ps,_),
	( Ps \== []; user:functions_exist(yes) ).


get_old_equations(Es) :-
	user:model_rules(Rs),
	rules_to_literals(Rs, Es).

rules_to_literals([], []).
rules_to_literals([(L => R) - _N | R1], [++equal(L, R)| R2]) :-
	rules_to_literals(R1, R2).



get_eq_from_constants([],_NewCon,[]).
get_eq_from_constants([C|R1], NewCon, [++equal(C, NewCon)|R2]) :-
	get_eq_from_constants(R1, NewCon, R2).





%---snip---snip------------------------------------------------------

analyze(SFin,SArm,NewCon) :-
	nl,
	write('Finite Model Building:'), nl,nl,
%	format(SFin,"~n",[]),

	user:model_domain(Dom),
	write('*** Domain:'), nl,
	format(SFin,"*** Domain:~n",[]),

	pprint_domain(SFin,Dom,0), nl,
	format(SFin,"~n",[]),

	user:model_signature(Psig, Fsig),
	constants_not_in_domain(Fsig, Dom, Cs),
	( Cs \== [] -> 
	    (write('*** Constants reducing to 0:'), nl, print(Cs), nl, nl,
	    format(SFin,"*** Constants reducing to 0:~n",[]),write(SFin,Cs),format(SFin,"~n~n",[]))
	;
		true
	),
	
        assert(user:functions_exist(not)),
	assert_modelfunctions(NewCon, Fsig),
	print_functions_if_present(SFin),

        write('*** Defined Predicates: '), print(Psig), nl,
	assert(user:predicates_exist(not)),
        assert_modelpredicates(Psig),

        tabulate_predicates(SFin),

	% extract gea/arm
	%write('<@<@<@ */'), nl,nl,
	%write('% @>@>@> extracted atomic representation <@<@<@:'), nl,nl,

	extract_arm(SArm,NewCon,Cs).


clear :-
	( model_show_exists -> abolish(user:model_show, 1) ; true),
	
	abolish(user:model_arm,1),
	abolish(user:model_domain,1),
	abolish(user:model_rules,1),
	abolish(user:model_setting,1),
	abolish(user:model_signature,2),
	abolish(user:model_orthset,1),
	%abolish(user:model_hus,1),

	( user:predicates_exist(yes) ->
        	findall(X1,user:modelpredicate(X1),All1),
        	retract_modelpredicates(All1),
		abolish(user:modelpredicate,1)
	;
		true
	),
	abolish(user:predicates_exist,1),

	( user:functions_exist(yes) ->
		findall(X,user:modelfunction(X),All),
		retract_modelfunctions(All),
		abolish(user:modelfunction,1)
	;
		true
	),
	abolish(user:functions_exist,1),

	%write('% EOF'), 
	nl.

% EOF
