:- module(herbrand,[domain_of_arm/3,
	remove_top_functor/2,
	signature/2,
	strip_of_arity/2,
	orthoset_of_arm/2,
	gamma_d/3,
	collect_domain/4]).

:- assertz(user:file_search_path(finMod,'.')).

:- use_module(library(lists)).
:- use_module(compatibility).

?- use_module(finMod(basic_preds)).


% from fin.pl

collect_domain(D1, D2, NC, D) :-
	append([NC], D1, D3),
	append(D3, D2, D).


/***********************
	signature(Terms,Sig)

	sucht alle Konstanten und Funktionssymbole aus der Liste 
	von Termen. Liefert eine Menge von  Funktor/Arity Termen.

************************/

signature1([],[]).
signature1([T|Ts], Rs) :-
	% var(T), should work but does not !
	mi_var(T), 
	signature1(Ts,Rs).
signature1([T|Ts], [F/N|Rest]) :-	
	functor(T,F,N),
	T =.. [F|Args],
	signature1(Args, H1),
	signature1(Ts, H2),
	append(H1,H2,Rest).

signature(A,B) :-
	signature1(A,C),
	list_to_set(C,B).

/***********************
	baseterms(Sig,Terms)

	liefert alle zu einer Signatur gehoerigen baseterms

************************/
baseterms([],[]).
baseterms([F/0|Sigs], [F|Baseterms]) :-
	baseterms(Sigs, Baseterms).
baseterms([F/N|Sigs], [T|Ts]) :-
	N > 0,
	length(SomeVars, N),
	T =.. [F|SomeVars],
	baseterms(Sigs,Ts).

%------------------------------------------
%
% strip_of_arity/2
%
%------------------------------------------

strip_of_arity([],[]).
strip_of_arity([F/_N|Sigs],[F|Fs]) :-
	strip_of_arity(Sigs,Fs).

/***********************
************************/
select_function(Function, Functor, Args) :-
	select_choice(Choice,Args),
	Function =.. [Functor|Choice]. 

/***********************
************************/
all_those_functions(Flist, Functor, Args) :-
	findall(X, select_function(X,Functor,Args), Flist).


/***********************
	coterms(Term, Baseterms, Cotermlist)

	generiert eine Liste der Coterme von Term bzgl. den Baseterms
	
************************/
coterms(T, Baseterms, Coterms) :-
	atom(T),	
	meta_select(Baseterms,T,H),
	copy_term(H,Coterms).
coterms(T, _Bt, [T1]):-
	mi_var(T), 
	copy_term(T,T1).
coterms(T, Baseterms, Coterms) :-
	functor(T,F,N),
	N > 0,
	T =.. [F|Args],
		% wir berechnen die cotermmengen aller argumente...
	coterms_of_list(Args, Baseterms, Cotermlist),
		% .. und fuegen die argumente selbst auch noch ein 
	prepend_to_listlist(Args,Cotermlist,CoAndArgs), 
		%  alle funktionsterme mit diesen argumenten ... 
	all_those_functions(H3,F,CoAndArgs),
		% ..sicherheitshalber noch ein set daraus machen; aber...
	list_to_set(H3,H4),
		% ...jetzt haben wir den urspr. term T wieder erzeugt! muss weg.
	meta_select(H4,T,Newterms),   
		% allgemeinen Term aus den baseterms entfernen	
	copy_term(T,T1), % achtung! MUSS kopiert werden
	copy_term(Baseterms, NewBaseterms),
	meta_select(NewBaseterms, T1, ReducedBt),  
	union(ReducedBt, Newterms, Coterms). 
	
coterms_of_list([],_,[]).
coterms_of_list([T|Ts], Bt, [Co|Cos]) :-
	coterms(T, Bt, Co),
	coterms_of_list(Ts, Bt, Cos).
	

/***********************
	relativation(A,B,C)

	liefert eine Liste C von Elementen aus A, die Instanzen von 
	B sind.

************************/
relativation(Coterms,S,Rel) :-
	copy_term(S,S1),
	findall(X,(member(X,Coterms), is_instance_of(X,S1)), Rel).


/***********************
************************/
orthogonalize_step(Terms, Bt, OrthExt) :- 
/*	write('*ostep: '), print(Terms), nl, */
	meta_select(Terms,A,H1),			% irgenein erster term,
	meta_select(H1,B,H2),			% und ein zweiter,
	\+ A == B,					% (aber nicht zweimal derselbe)
	unify(A,B,CommInst),		% sollen unifizierbar sein;
	append(H2,[CommInst],H3),  
	coterms(CommInst, Bt, Coterms), 
	relativation(Coterms, A, Set1),
	relativation(Coterms, B, Set2),
	append(H3, Set1, H4),	
	append(H4, Set2, OE),
	list_to_set_struct(OE,OrthExt),
	 !.

/***********************
************************/
orthogonalize(Terms, Bt, Terms) :-
	\+ orthogonalize_step(Terms, Bt, _).
orthogonalize(Terms, Bt, OrthExt) :-
	orthogonalize_step(Terms, Bt, H), 
	orthogonalize(H, Bt, OrthExt).


remove_top_functor([],[]).
remove_top_functor([A|As],Bs) :-
	A =.. [_|Args],
	remove_top_functor(As,H),
	append(Args,H,Bs).

collect_nonvar_subterms([],[]).
collect_nonvar_subterms([T|Ts],Rs) :-
	mi_var(T),
	collect_nonvar_subterms(Ts,Rs).
collect_nonvar_subterms([T|Ts],[T|Rs]) :-
	\+ mi_var(T),	
	T =.. [_F|Args],
	copy_term(Args, NewArgs),
	collect_nonvar_subterms(NewArgs,Subterms),
	collect_nonvar_subterms(Ts,H),
	append(Subterms,H,Rs).


termset_of_arm(Arm,Termset) :-
	remove_top_functor(Arm,T1),
	collect_nonvar_subterms(T1,T2),
	list_to_set_struct(T2,Termset).

orthoset_of_arm(Arm,Ortho) :-
	termset_of_arm(Arm,Ts),
	signature(Ts,HUs),
	baseterms(HUs,Bt),
	orthogonalize(Ts,Bt,Ortho).


gamma_d(_NewCon,[],[]).
gamma_d(NewCon,[T|Ts],[R|Rs]) :-
	bind_vars_to_constant(T,NewCon,R),
	gamma_d(NewCon,Ts,Rs).


domain_of_arm(Arm,NC,[NC|Domain]) :-
	orthoset_of_arm(Arm,Oset),
	gamma_d(NC,Oset,Domain).



