#! /usr/bin/sh

# installation shell to set up all
# necessary program names and directories
# for TILT

# Procedure calls have to be changed
# in 
#    tilt -> python
#    tilt.py  -> spass, otter2, sicstus, pl
#    arm_tilt.pl -> otter2
#    tptp2X -> TILT home directory

oldIFS="$IFS"
IFS=":"
path=`echo $PATH`
IFS="$oldIFS"
path="$path /usr/local/bin /usr/local/sbin /usr/bin /usr/sbin /bin /sbin 
    /pd/mb/fwf/hans /pd/mb/fwf/hans/otter2 /pd/mb/fwf/georg /pd/mb/fwf/mandana /pd/mb/fwf/gernot"
ucmds="false true awk basename chmod cp dirname grep mkdir pwd rm sed"
binaries="python spass otter2 pl sicstus"
directories="bin top classify standard exper finmodel cleval TPTP2X examples"
tiltfiles="Readme install.sh deinstall.sh"
tilt_exe="tilt"

tilt_inst=/usr/local/tilt

set_cmd(){
  for c in $*
  do
    notfound=$true
    for p in $path
    do
      if [ -x $p/$c -a ! -d $p/$c ]
      then
        eval $c=$p/$c
        notfound=$false
        break
      fi
    done
    if $notfound
    then
      eval $c=$c
      return 1
    fi
  done
  return 0
}

read_dir(){
  while $true
  do
    eval echo -n \"Directory for $1 [\$$2]: \"
    read a
    a=`echo $a | $sed 's|~|'$HOME'|;s|$HOME|'$HOME'|'`
    if [ ! "$a" ]
    then
      eval a=\$$2
    fi
    if (echo $a | $grep -q '^/')
    then
      :
    else
      a=`$pwd`/$a
    fi
    if [ -d $a ]
    then
      echo "    Found directory $a"
      eval $2=$a
      break
    else
      echo "    Create directory $a"
      mkdir $a
      eval $2=$a
      break
    fi
  done
}

read_loc_bin(){
  while $true
  do
    if [ $2 ]; then
	eval echo -n \"Full path for $1 [\$$2]: \"
	read a
	a=`echo $a | $sed 's|~|'$HOME'|;s|$HOME|'$HOME'|'`
	if [ ! "$a" ]; then
	    eval a=\$$2
	fi
    else
	eval echo -n \"Location for $1 []: \"
	read a
	a=`echo $a | $sed 's|~|'$HOME'|;s|$HOME|'$HOME'|'`
    fi
    aa=`echo "$a" | $sed 's|^./||'`
    if (echo $aa | $grep -q '^/'); then
      :
    else
      aa=`$pwd`/$aa
    fi
    if [ -x $aa -a ! -d $aa ]
    then
      eval $1=$aa
      echo "   Found: $aa"
      break
    else
      echo "   Error: $aa does not exist or has no execute permission."
    fi
  done
}

# Searching for some Unix commands
if set_cmd $ucmds
then
  :
else
  echo "Error: could not find Unix command $c."
  exit 1
fi

# Searching for Executables
for x in $binaries
do
    if set_cmd $x; then
	read_loc_bin $x $x
    else
	read_loc_bin $x 
    fi
done

distdir=`$dirname $0`
cd $distdir
distdir=`$pwd`

read_dir 'TILT installation directory' tilt_inst

sedscript=$distdir/sedscript
$rm -f $sedscript
for x in $binaries tilt_inst
do
  eval echo "s\|@$x@\|\$$x\|g" >> $sedscript
done

echo "Installing ..."

mode=644
for f in $tiltfiles
do
    $sed -f $sedscript $f >$tilt_inst/$f
    $chmod $mode $tilt_inst/$f
done

mode=644
for d in $directories
do
    if [ $d = "bin" ]; then
	mode=755;
    else 
	mode=644;
    fi
    echo "Change directory to $distdir/$d"
    cd $distdir/$d
    if [ -d $tilt_inst/$d ]; then
	:
    else
	echo "Create directory $tilt_inst/$d"
	$mkdir $tilt_inst/$d
    fi

    for f in *
    do
	$sed -f $sedscript $f >$tilt_inst/$d/$f
	$chmod $mode $tilt_inst/$d/$f
    done
done

$rm -f $sedscript

echo ""
echo "Installation done."
echo ""
echo "In case of problems, send e-mail to mb@logic.at."