%------------------------------------------------------
% arm computation
%
% new version
%------------------------------------------------------
:- module(arm_tilt, [arm/1]).

:- assertz(user:file_search_path(finMod,'../finmodel')).
:- assertz(user:file_search_path(tptp,'.')).

:- use_module(library(system)).
:- use_module(library(lists)).
?- ensure_loaded(tptp(tptp_converter)).
%?- use_module(finMod(fin)).
:- use_module(finMod(basic_preds),[write_arm/2]).

%-------------------------------------------------------
% msc

is_unit_clause([_L|[]]).

is_positive_clause([]).
is_positive_clause([L|Ls]) :-
	\+ L = --_,
	is_positive_clause(Ls).

is_positive_unit_clause(C) :-
	is_positive_clause(C),
	is_unit_clause(C).

is_instance(_,[]) :- !, false.
is_instance(L,[L|_]) :- !.
is_instance(L,[_|R]) :-
	is_instance(L,R).

% read clauses
reading_clauses(Tfile, Ls) :-
	open(Tfile, read, Tstream),
	reading_cl(Tstream, Ls),
	close(Tstream).
reading_cl(T, [Sig_clause | Sig]) :-
	read(T, Clause),
	Clause \== end_of_file,
	!,
        numbervars(Clause,0,_),
        filter_cl(Clause, Sig_clause),
	reading_cl(T, Sig).		
reading_cl(_,[]).
filter_cl(input_clause(_,_,C), C).


swritef(S,Suffix,Fname) :-
	name(Suffix, Suffixlist),
	name(Fname, Fnamelist),
	append(Fnamelist, Suffixlist, Slist),
	name(S, Slist).

basename_ptmp(Tfile, Bname) :-
	name(Tfile, Tfilelist),
	baselist(Tfilelist, Baselist),
	name(Bname, Baselist).

baselist(Tfilelist, Baselist) :-
	append(Baselist, ".tmp.p", Tfilelist), !.
baselist(Tfilelist, Baselist) :-
	append(Baselist, ".p", Tfilelist), !.
baselist(Tfilelist, Tfilelist).

% naming conventions for renamings not very clear, ad hoc help:
% ( .tmp.p and .p already stripped)
basename_pvdpvdeq(Tfile, Bname) :-
	name(Tfile, Tfilelist),
	baselist2(Tfilelist, Baselist),
	name(Bname, Baselist).
baselist2(Tfilelist, Baselist) :-
	append(Baselist, ".pvd", Tfilelist), !.
baselist2(Tfilelist, Baselist) :-
	append(Baselist, ".pvdeq", Tfilelist), !.
baselist2(Tfilelist, Tfilelist).


%-------------------------------------------------------

% if the file F contains Box, then as first clause
is_satisfiable(F) :-
	seeing(Oldinput),
	see(F),	
	read(Firstclause),
	seen,
	see(Oldinput),
	\+ Firstclause = input_clause(_,_,[]).	


% stable form of clause set in Tfile -> Tfiletmp
fixed_point(Tfile, Tfiletmp) :-
	% Tfile suffix .p or .tmp.p
	basename_ptmp(Tfile,Bname),
	swritef(Tfiletmp, '.tmp.p', Bname),
	swritef(Otterin, '.otter.in',Bname),
	swritef(Otterout, '.otter.out',Bname),
	tptp2otter(Tfile,Otterin),
	% gnerate shell command
	name(Otterin, Otterinlist),
	name(Otterout, Otteroutlist),
	append("@otter2@ <",Otterinlist, S1),
	append(S1, " >", S3),
	append(S3, Otteroutlist, S4),
	name(S, S4),
	% shell command
	write('... invoking otter'), nl,
	( shell(S) -> true; true ),
	otter2tptp(Otterout, Tfiletmp).
	

% arm computation
% finite model building is called seperately by
% tilt

arm(Tfile) :-
	fixed_point(Tfile, Tfiletmp),
	% unsatisfiablility of Tfile should be obvious
	% regarding the computed arm
	basename_ptmp(Tfile, Bname),
	swritef(Tfiletmptmp, '.tmp.tmp.p', Bname),
	alpha(Tfiletmp, Tfiletmptmp),
	!,
	%write(Tfiletmptmp),
	rename_file(Tfiletmptmp, Tfiletmp),
	arm(Tfiletmp).
arm(Tfile) :-
	% positive clauses of Tfile are unit
	% produce output
	basename_ptmp(Tfile, Bname),
	swritef(Tfiletmp, '.tmp.p', Bname),

	is_satisfiable(Tfiletmp), 
	!,
	write('... model found.'), nl, nl,

	basename_pvdpvdeq(Bname, B2),
	%swritef(Settingfile, '.ren', B2),
	%swritef(Finfile, '.fin', B2),
	%swritef(NewArmfile, '.arm', B2),
	%swritef(Problemfile, '.p', B2),
	swritef(Xfile, '.ren.arm', B2),

	% filtration and output in arm
	beta(Tfiletmp, Xfile),
	write('--------------------------------------------------------'),nl,
	write('(renamed) ARM filed in '''), write(Xfile), write(''':'), nl,
	plot_on_screen(Xfile), nl.
arm(Tfile) :-
	% file is unsatisfiable
	basename_ptmp(Tfile, Bname),
	basename_pvdpvdeq(Bname, B2),
	swritef(Problemfile, '.p', B2),
	write('The clause set in '''),
	write(Problemfile), write(''' is unsatisfiable.'), nl,
	cleanup(Bname).

cleanup(Bname) :-
	% delete temporary files used in arm computation
	swritef(Tmpfile, '.tmp.p',Bname),
	( file_exists(Tmpfile) ->
	delete_file(Tmpfile); true ),
	swritef(TTmpfile, '.tmp.tmp.p',Bname),
	( file_exists(TTmpfile) ->
	delete_file(TTmpfile); true ),
	% delete temporary files used in fixpoint constrution
	swritef(Otterin, '.otter.in',Bname),
	( file_exists(Otterin) ->
	delete_file(Otterin); true ),
	swritef(Otterout, '.otter.out',Bname),
	( file_exists(Otterout) ->
	delete_file(Otterout); true ).

%------------------------------------------------------

plot_on_screen(Xfile) :-
	name('cat ', X1),
	name(Xfile, X2),
	append(X1, X2, X12),
	name(Sys, X12),
	system(Sys).	

arm_header(Sfile) :-
	write('% @>@>@> atomic model representation <@<@<@:'), nl,
	write('% setting: '), 
	reading_setting(Sfile, S),
	print(S), nl,nl.
/*
arm_unsat_epilog :-
	write('% ARM does not exist: '),
	write(' clause set is unsatisfiable.'),
	% nl,
	% write('input_clause(Box, theorem, []).'),
	nl.
*/

% beta ... filtration for positive unit clauses
% patch, tilt 0.1 doesn't allow comments
% in clause eval, hence no arm_header


%beta(Tfile, Xfile, _Sfile) :-
%	reading_clauses(Tfile, Cs),	
%	beta_filter(Cs, As),
%	sort(As, Arm),
%	open(Xfile,write,XfileStream),
%	write('[printing '), write(Xfile),
%	write(']'),
%	nl, 
%	arm_header(Sfile),
%	write_arm(XfileStream,Arm),
%	close(XfileStream).
%	write('% EOF'), 
%	nl.


beta(Tfile, Xfile) :-
	reading_clauses(Tfile, Cs),	
	beta_filter(Cs, As),
	sort(As, Arm),
	open(Xfile,write,XfileStream),
	write('[printing '), write(Xfile),
	write(']'),
	nl, 
	write_arm(XfileStream,Arm),
	close(XfileStream).

beta_filter([], []).
beta_filter([C|R], As) :-
	beta_filter2(C, Cunit),
	beta_filter(R, As2),
	append(Cunit, As2, As).
beta_filter2([++equal(A,A)], []) :- !.
beta_filter2(C, C) :- is_positive_unit_clause(C), !.
beta_filter2(_, []).	
		
/* [old]
beta(Tfile, Xfile, Sfile) :-
	telling(Oldoutput),
	tell(Xfile),
	seeing(Oldinput),
	see(Tfile),
	arm_header(Sfile),
	repeat,
		read(Clause),
		( Clause == end_of_file ->
			true
		;
			numbervars(Clause,0,_),
			beta_process(Clause),
			fail
		),
	!,
	write('% EOF'),
	seen,
	see(Oldinput),
	told,
	tell(Oldoutput).
	

beta_process(input_clause(_, _, [++equal(A,A)])) :- !.
%beta_process(input_clause(Name, Type, C)) :- 
%	is_positive_unit_clause(C), !,
%	write(input_clause(Name, Type, C)),
%	write('.'),
%	nl.
beta_process(input_clause(_, _, C)) :- 
	is_positive_unit_clause(C), !,
	write(C),
	nl.
beta_process(input_clause(_, _, [])) :- 
	arm_unsat_epilog.
*/

% alpha ... reduction of a positive non-unit clause

alpha(Tfile, Xfile) :-
	telling(Oldoutput),
	tell(Xfile),
	seeing(Oldinput),
	see(Tfile),
	repeat,
		read(Clause),
		( Clause == end_of_file ->
			true
		;
			numbervars(Clause,0,_),
			alpha_process(Clause)
		),
	!,
	alpha1(Oldoutput, Oldinput, Clause).

alpha1(Oldoutput, Oldinput, Clause) :-
	% no positive non-unit clause
	Clause == end_of_file,
	seen,
	see(Oldinput),
	told,
	tell(Oldoutput),
	!,
	fail.
alpha1(Oldoutput, Oldinput, _ ) :-
	% positive non-unit clause was found and reduced
	% copy rest of clauses
	repeat,
		read(Clause),
		( Clause == end_of_file ->
			true
		;
			numbervars(Clause,0,_),
			write(Clause),
			write('.'),
			nl,
			fail
		),

	!,
	seen,
	see(Oldinput),
	told,
	tell(Oldoutput).


alpha_process(input_clause(Name, Type, [C1 | R])) :- 
	\+ is_unit_clause([C1 | R]),
	is_positive_clause([C1 | R]),
	% reduction
	write(input_clause(Name, Type, [C1])),
	write('.'),
	nl.	
	% true 
alpha_process(input_clause(Name, Type, C)) :- 
	write(input_clause(Name, Type, C)),
	write('.'),
	nl,
	fail.	


% EOF
