:- [mb].
:- use_module(library(ordsets)).

paths([], []).
paths([Clause | Clauses], Paths) :-
    paths(Clauses, SubPaths),
    append_literals(Clause, SubPaths, Paths).

append_literals([], _Paths, []).
append_literals(Literals, [], Paths):-
    make_paths(Literals, Paths).
append_literals([Literal | Literals], SubPaths, Paths):-
    append_literal(Literal, SubPaths, Paths1),
    append_literals(Literals, SubPaths, Paths2),
    append(Paths1, Paths2, Paths).

append_literal(Literal, [Path], [[Literal | Path]]).
append_literal(Literal, [Path | Paths], [ [Literal | Path] | Paths1]):-
    append_literal(Literal, Paths, Paths1).

make_paths([], []).
make_paths([Literal | Literals], [[Literal] | Paths]) :-
    make_paths(Literals, Paths).

satisfiable([Clause | _Clauses], Clause) :-
    \+ is_tautology(Clause).
satisfiable([_Clause | Clauses], Model) :-
    satisfiable(Clauses, Model).

find_model(Clauses, Model):-
    invert_literals(Clauses, Clauses1),
    paths(Clauses1, Paths),
    satisfiable(Paths, Model1),
    list_to_ord_set(Model1, Model).

invert_literals([], []).
invert_literals([Clause | Clauses], [NewClause | NewClauses]) :-
	invert_literals_in_clause(Clause, NewClause),
	invert_literals(Clauses, NewClauses).

invert_literals_in_clause([], []).
invert_literals_in_clause([Lit | Lits], [NewLit | NewLits]) :-
	invert_literal(Lit, NewLit),
	invert_literals_in_clause(Lits, NewLits).

invert_literal(++Atom, --Atom).
invert_literal(--Atom, ++Atom).
