% ==============================================================
% conversion from an otter-outputfile to a file in tptp-syntax.
% to run use entry 1!
% temporary file employed:
% otter2tptp(<otteroutputfilename>,<tptpfilename>) uses the 
% temporary file '<otteroutputfilename>.tmp'
% ==============================================================

:- assertz(user:file_search_path(tptp_m,'.')).

:- op(100,fx,--).
:- use_module(library(system)).
:- use_module(library(lists)).

?- ensure_loaded(tptp_m('m_tptp2X.main')).

%---------------------------------------
% msc

% name of temporary file is computed.
temporary_file(Otterfile,Tmpfile) :-
	name(Otterfile,Otterfilelist),
	append(Otterfilelist,".tmp",Tmpfilelist),
	name(Tmpfile,Tmpfilelist).



write_header(F1, F2) :-
	write('%---------------------------------------------'), nl,
	write('% otter2tptp produced transformation from'), nl,
	write('% otter-outputfile '''), 
	write(F1),
	write(''' to'), nl,
	write('% tptp-file  '''), 
	write(F2),
	write('''.'), nl,
	write('%---------------------------------------------'), nl,
	nl, nl.

%---------------------------------------
% main

% entry 1 * * * * *
otter2tptp(Otterfile, Tptpfile) :-
	telling(Oldoutput),
	tell(Tptpfile),
	write_header(Otterfile, Tptpfile),
	process_file(Otterfile),
	told,
	tell(Oldoutput),
	fail.
otter2tptp(O,_) :-
	temporary_file(O,T),
	(file_exists(T) ->
	delete_file(T); true).

% entry 2 * * * * * for testing
process_file(Otterfile) :-
	seeing(Oldinput),
	see(Otterfile),
	repeat,
		read_line(Line),
		(Line == end_of_file ->
			true
		;			
			process_line(Otterfile,Line)
		),
	!,
	seen,
	temporary_file(Otterfile,Tempfile),
	transform(Tempfile),
	see(Oldinput).


%---------------------------------------
% filter information


process_line(Otterfile,'---------------- PROOF ----------------') :-
	temporary_file(Otterfile,T),
	telling(Oldoutput),
	tell(T),
	write('[[48/*0*/],[]].'), nl,
	told,
	tell(Oldoutput).

process_line(Otterfile,'Search stopped because sos empty.') :-
	temporary_file(Otterfile,T),
	telling(Oldoutput),
	tell(T),
	find_text('list(usable).'),
	read_clauses,
	told,
	tell(Oldoutput).


%------------------------------------------
% filter clauses to tmp-file


find_text(Text) :-
	repeat,
		read_line(Line),
		(Line == end_of_file; Line == Text),
	!,
	(Line == end_of_file ->
		fail
	;
		true
	).

read_clauses :-
	repeat,
		read_line(Line),
		((Line == end_of_file; Line == 'end_of_list.') ->
			true
		;
			cut_line(Line, Clause, Commentary),
			transform_to_list(Clause, Clause_as_list),
			name(Commentary, Commlist),
			write([Commlist, Clause_as_list]), write('.'), nl,
			fail
		),
	!.		

cut_line(Line,Clause, Commentary) :-
	name(Line, Linelist),
	cut_list(Linelist, Clauselist),
	cut_list2(Linelist, Commlist),
	name(Clause, Clauselist),
	name(Commentary, Commlist).
	

cut_list([], []).
cut_list([93/*]*/, 32/* */ | L], L) :-  !.
cut_list([_ | L], M) :- cut_list(L, M).

cut_list2([],[]).
cut_list2([32/* */ | _], []) :- !.
cut_list2([X | L], [X | M]) :- cut_list2(L,M).


transform_to_list(Line, Otterlist) :-
	name(Line, List),
	transform_to_list2(List, Otterlist2),
	name(Otterlist, [ 91/*[*/ | Otterlist2]).

transform_to_list2([46/*.*/], [93/*]*/]) :- !.
transform_to_list2([],[]) :- !.

transform_to_list2([ 124/*|*/ | Rest1], [ 44/*,*/ | Rest2]) :-
	!,
	transform_to_list2(Rest1, Rest2).

transform_to_list2([ 45/*-*/ | Rest1], [ 45, 45/*-*/ | Rest2]) :-
	!,
	transform_to_list2(Rest1, Rest2).

transform_to_list2([ X | Rest1], [ X | Rest2]) :- 
	transform_to_list2(Rest1, Rest2).
	
	
%----------------------------------------------
% tmp-file to tptp-file

transform(Tempfile) :-
	seeing(Oldinput),
	see(Tempfile),
	repeat,
		read(Info),
		(Info == end_of_file ->
			true
		;			
			process_clause(Info),
			fail
		),
	!,
	seen,
	see(Oldinput).

process_clause([Commlist, Clause]) :-
	write('input_clause(otter_clause_'),
	name(Commentary,Commlist),
	write(Commentary),
	write(', axiom,'), nl,
	write('	['),
	numbervars(Clause,0,_),
	write_clause(Clause),
	write(']).'), 
	nl,nl.

write_clause([]).
write_clause([X]) :-
	!,
	write_literal(X).
write_clause([X|R]) :-
	write_literal(X),
	write(','),
	write_clause(R).

write_literal(--X) :-
	!,
	write('--'),
	write_literal2(X).
write_literal(X) :-
	write('++'),
	write_literal2(X).
	
write_literal2(X=Y) :-
	!,
	write('equal('),
	write(X),
	write(','),
	write(Y),
	write(')').
write_literal2(X) :-
	write(X).

% ==============================================================
% conversion from a tptp-file to an otter input-file.
% employs tptp2X()
% 
% ==============================================================



clauses_in_pvd(Tfile) :-
	% file name suffix .pvd.p or .pvd.tmp.p
	name(Tfile, Tfilelist),	
	( append(_ , ".pvd.p", Tfilelist) ;
	  append(_ , ".pvd.tmp.p", Tfilelist) ).
	

filerenaming(Tptpfile, Otterfile) :-
	% by tptp2X produced output filename...
	name(Tptpfile, Tptpfilelist),
	append(Baselist, [46, 112 /* .p */], Tptpfilelist),
	append(Baselist, "+rm_eq_stfp.in", Oldfilelist),
	name(Oldfile, Oldfilelist),
	% gets renamed
	% write(Oldfile),
	rename_file(Oldfile, Otterfile),
	!.
filerenaming(_,_).

% *** problem with tptp2X: too short filenames are not accepted!!!!
% ad hoc help:
tptp2otter(Tptpfile, Otterfile) :-
	checkname1(Tptpfile, Tptpfile1),
	tptp2otter1(Tptpfile1, Otterfile),
	checkname2(Tptpfile, Tptpfile1).

checkname1(T1, T2) :-
	name(T1, Ts1),
	length(Ts1, N),
	N < 8,
	!,
	append(Ts1, "xxx.p", Ts2),
	name(T2, Ts2),
	rename_file(T1, T2).
checkname1(T1, T1).

checkname2(T1, T2) :-
	name(T1, Ts1),
	length(Ts1, N),
	N < 8,
	!,
	rename_file(T2, T1).
checkname2(_, _).
% ***

% gm
% ugly hack replacing current directory
% by /tmp/.
% ugly but works!

tptp2otter1(Tptpfile, Otterfile) :-
	% clause set is in pvd
	clauses_in_pvd(Tptpfile), 
	!,
	tptp2X(Tptpfile,
		[],
		[rm_equality:stfp],
		[otter:all:[
			set(hyper_res),
			clear(order_hyper),
			set(process_input),
			set(factor),
			set(for_sub),
			set(back_sub),
			set(print_lists_at_end)
			]
		],
		'/tmp/.'),
%		see(Tptpfile),
		close(Tptpfile), /* Tptpfile not closed in tptp2X !!! */
		filerenaming(Tptpfile, Otterfile).

tptp2otter1(Tptpfile, Otterfile) :-
	% by default: clause set expected to be in pvd=
	tptp2X(Tptpfile,
		[],
		[rm_equality:stfp],
		[otter:all:[
			set(p_res),
			set(process_input),
			set(factor),
			set(para_p_into),
			set(para_p_from),
			set(para_from_left),
			clear(para_from_right),
			set(para_into_right),
			set(para_into_left),
			set(para_from_vars),
			set(order_eq),
			set(lrpo),
			set(for_sub),
			set(back_sub),
			set(dynamic_demod),
			set(back_demod),
			set(print_lists_at_end)
			]
		],
		'/tmp/.'),
%		see(Tptpfile),
		close(Tptpfile), /* Tptpfile not closed in tptp2X !!! */
		filerenaming(Tptpfile, Otterfile).




tptp2otter(Tptpfile) :-
	name(Tptpfile, Tptpfilelist),
	append(Baselist, [46, 112 /* .p */], Tptpfilelist),
	append(Baselist, ".otter.in", Otterfilelist),
	name(Otterfile, Otterfilelist),
	tptp2otter(Tptpfile, Otterfile),
	!.
tptp2otter(Tptpfile) :-
	name(Tptpfile, Tptpfilelist),
	append(Tptpfilelist, ".otter.in", Otterfilelist),
	name(Otterfile, Otterfilelist),
	tptp2otter(Tptpfile, Otterfile).
