from Tkinter import *
from cledit import ClauseEditor
from ScrolledText import ScrolledText
from formula import Formula
import string

class FormulaEditor:

	def __init__(self, text):
		self.text = text
		self.text.bind("<Up>", self.up)
		self.text.bind("<Down>", self.down)
		self.num = 0
		self.current = 0

	def show_text(self, lines, type = "unknown"):
		self.type = type
		self.text.insert("1.0", string.join(lines))
		if self.type == 'tptp':
			self.formula = Formula(lines)
			(start, end) = self.formula.get_clause_pos(self.current)
			self.text.tk.call('tkTextSetCursor', self.text._w, end)
			self.text.tk.call('tkTextSetCursor', self.text._w, start)
			self.highlight_comments()
			self.highlight_clause(self.current)


	def highlight_comments(self):
		if self.type == 'tptp':
			comment_positions = self.formula.GetCommentPositions()
			for comment in comment_positions:
				self.text.tag_add("comment", comment[0], comment[1])
			self.text.tag_config("comment", foreground="blue")
	
	def add_clause(self):
		"""Calls the ClauseEditor-Dialog and extracts a
		TPTP-Formula from it."""

		if self.type == 'tptp':
			a,b,c = ClauseEditor(self, self.num).show()
			thestring = "input_clause("+a+", "+b+", [" +c+"]).\n\n"
			self.text.insert("insert", thestring)
			self.num=self.num+1
			thetext = self.text.get('1.0', "end")
			lines = string.splitfields(thetext, '\n')
			self.formula = Formula(lines)
			(start, end) = self.formula.get_clause_pos(self.current)
			self.text.tk.call('tkTextSetCursor', self.text._w, end)
			self.text.tk.call('tkTextSetCursor', self.text._w, start)
			self.highlight_clause(self.current)

	def clear_contents(self):
		self.text.delete("0.0", END)
		self.text.focus_set()		
		self.current = 0
		self.num = 0


	def edit_clause(self):
		if self.type == 'tptp':
			(name, type, contents) = self.formula.get_description(
								self.current)
			a,b,c = ClauseEditor(self, self.num, (name, type,
						contents)).show()
			thestring = "input_clause("+a+", "+b+", [" +c+"]).\n\n"
			self.text.insert("insert", thestring)
			self.num=self.num+1
			thetext = self.text.get('1.0', "end")
			lines = string.splitfields(thetext, '\n')
			self.formula = Formula(lines)
			(start, end) = self.formula.get_clause_pos(self.current)
			self.text.tk.call('tkTextSetCursor', self.text._w, end)
			self.text.tk.call('tkTextSetCursor', self.text._w, start)
			self.highlight_clause(self.current)

	def remove_clause(self):
		pass
     
	def add_comment(self):
		pass

	def edit_comment(self):
		pass

	def remove_comment(self):
		pass

	def up(self, event):
		if self.type == 'tptp':
			if self.current>0:
				self.current = self.current - 1
			(start, end) = self.formula.get_clause_pos(self.current)
			self.text.tk.call('tkTextSetCursor', self.text._w, end)
			xy=string.splitfields(start, ".")
			newstart=str(eval(xy[0])+1)+"."+xy[1]
			self.text.tk.call('tkTextSetCursor', self.text._w, 
					  newstart)
			self.highlight_clause(self.current)

     
	def down(self, event):
		if self.type == 'tptp':
			if self.current < self.formula.num_of_clauses()-1:
				self.current = self.current + 1
			(start, end) = self.formula.get_clause_pos(self.current)
			self.text.tk.call('tkTextSetCursor', self.text._w, end)
			xy=string.splitfields(start, ".")
			newstart=str(eval(xy[0])-1)+"."+xy[1]
			self.text.tk.call('tkTextSetCursor', self.text._w, 
					  newstart)
			self.highlight_clause(self.current)

	def highlight_clause(self, number):
		if self.type == 'tptp':
			(start, end) = self.formula.get_clause_pos(number)
			self.text.tag_delete("current clause")
			self.text.tag_add("current clause", start, end)
			self.text.tag_configure("current clause", background="white", relief="sunken")






