from Tkinter import *
from ScrolledText import ScrolledText
import string

class ComponentEditor:
	"""presents a dialog in which the user can determine the 
		contents of a component"""

	def __init__(self, parent=None, name="", contents=()):
		self.top = Toplevel(parent)
		self.top.title("Edit " + name)
		self.top.withdraw()
		frame = Frame(self.top)
		frame.pack(expand=1, fill=BOTH)
		self.textvars = []
		for element in contents:
			content_frame = Frame(frame)
			content_frame.pack(side=TOP)
			label = Label(content_frame, text=element[0]+":")
			label.pack(side=LEFT, anchor=NE)
			textvar = StringVar()
			textvar.set(element[1])
			editfield = self._get_editfield(content_frame, textvar)
			self.textvars.append(textvar)
			editfield.pack(side=RIGHT, anchor=NW)
		button_frame = Frame(frame)
		button_frame.pack(side=TOP)
		ok_button = Button(button_frame, text="OK", command=self.do_ok)
		ok_button.pack(side=LEFT)
		cancel_button = Button(button_frame, text="Cancel", 
							   command=self.do_cancel)
		cancel_button.pack(side=RIGHT)

	def _get_editfield(self, parent, aString):
		return EditEntry(parent, aString)

	def do_ok(self):
		self.ok = 1
		self.top.destroy()
		
	def do_cancel(self):
		self.ok = 0
		self.top.destroy()

	def show(self):
		""" shows the dialog and waits for the user to
		terminate it again (wait_window). returns the
		name, type and contents of the clause"""

		self.top.deiconify()
		self.top.wait_window()
		return self.ok
		
	def get_result(self):
		result = []
		for textvar in self.textvars:
			result.append(textvar.get())
		return result
		
class EditEntry (Entry):
	def __init__(self, parent, textvar):
		Entry.__init__(self, parent)
		self.config(textvariable=textvar)

	def get_contents():
		return self.textvar.get()
		
if __name__ == "__main__":
	tk = Tk()
	comped = ComponentEditor(tk, (("Name", "abc"), ("Type", "def")))
	if comped.show():
		print "Result:", comped.get_result()
	else:
		print "Dialog canceled!"
