from Tkinter import *
import compon
import comped
import file
import string

class FileEditor:

    def __init__(self, text):
        self.text = text # This is the text area
        self.text.bind("<Up>", self.up)
        self.text.bind("<Down>", self.down)

    def show_text(self, lines, type = "unknown"):
        self.type = type
        if self.type == 'tptpc' or self.type == 'tptpf':
            self.file = file.TPTPClauseFile(lines)
        elif self.type == "cnf" or self.type == "dfg":
            self.file = file.SPASSClauseFile(lines)
        elif self.type == "otter":
            self.file = file.OtterFile(lines)
        else:
            self.file = file.File(lines)
        self.text.insert("1.0", `self.file`)
        (start, end) = self.file.get_current_delimiters()
        self.text.mark_set('insert', start)
        self.highlight_comments()
        self.highlight_current_component(-1)

    def highlight_comments(self):
        comment_positions = self.file.get_comment_positions()
        for comment in comment_positions:
            self.text.tag_add("comment", comment[0], 
                              comment[1])
        self.text.tag_config("comment", foreground="blue")
    
    def clear_contents(self):
        self.text.delete("1.0", END)
        self.text.focus_set()        
        self.current = 0

    def add_component(self):
        typechoice = TypeChoice(None, self.file.types)
        if typechoice.show():
            type, name, contents = typechoice.get_result()
            ce = comped.ComponentEditor(None, name, contents)
            if ce.show():
                result = ce.get_result()
                comp = type([""])
		print result
		comp.config(result)
                self.file.add_component(comp)
                self.clear_contents()
                self.text.insert("1.0", `self.file`)
                self.highlight_comments()
                self.highlight_current_component(self.current)

    def edit_component(self):
        contents = self.file.get_current_description()
        ce = comped.ComponentEditor(None, "Component", contents)
        if ce.show():
            result = ce.get_result()
	self.file.remove_component()
        self.file.add_component(result)
        self.clear_contents()
        self.text.insert("1.0", `self.file`)
        self.highlight_comments()
        self.highlight_current_component(self.current)

    def remove_component(self):
        pass
     
    def up(self, event):
        self.file.go_up()
        self.highlight_current_component(1)
     
    def down(self, event):
        self.file.go_down()        
        self.highlight_current_component(-1)

    def highlight_current_component(self, offset_for_cursor):
        (start, end) = self.file.get_current_delimiters()
        self.text.tag_delete("current component")
        self.text.tag_add("current component", start, end)
        self.text.tag_configure("current component", 
                background="white", relief="sunken")
        startindices = string.splitfields(start, ".")
        start = `string.atoi(startindices[0]) + offset_for_cursor` + \
            "." + startindices[1]
        self.text.mark_set('insert', start)

class TypeChoice:
    def __init__(self, parent=None, types=()):
        """Allows to choose a type of component - input-clause, 
           comment, etc
           types: [
                   (type1, name1, [arg11, arg12, arg13]),
                   (type2, name2, [arg21]),
                   (type3, name3, [arg31, arg32, arg33])
                  ]
        """
        self.top = Toplevel(parent)
        self.top.title("Choose type")
        self.types = types
        frame = Frame(self.top)
        frame.pack(side=TOP)
        Label(frame, text="Choose type of component").pack()
        print types
        for idx in range(len(types)):
            print idx
            button = Button(frame, text=types[idx][1])
            button.configure(command=lambda self=self, command=idx: 
                             self.exit(command))
            button.pack(side=TOP, expand=1, fill=X)
        button = Button(frame, text="Cancel", command=self.cancel)
        button.pack(side=TOP)

    def show(self):
        """ shows the dialog and waits for the user to
        terminate it again (wait_window). returns the
        name, type and contents of the clause"""

        self.top.deiconify()
        self.top.wait_window()
        return not self.cancel
           
    def cancel(self):
        self.cancel = 1
        self.top.destroy()
        
    def exit(self, command):
        self.choice = command
        self.cancel = 0
        self.top.destroy()
        
    def get_result(self):
        print self.types[self.choice]
        return self.types[self.choice]

if __name__ == "__main__":
    tk = Tk()
    tc = TypeChoice(tk, 
                      (
                        (compon.TPTPClause, "TPTP Clause", 
                            (("Name", ""), ("Type", ""), ("Contents", ""))),
                        (compon.Comment, "Comment", (("Comment", ""),)),
                        (compon.SPASSClause, "Clause", (("Stuff", ""),))
                      )
                   )
    if tc.show():
        choice = tc.get_result()
        ce = comped.ComponentEditor(tk, choice[1], choice[2])
        if ce.show():
            print ce.get_result()
  
