#!/usr/local/bin/python
import sys
import os
import string
import glob
import tkMessageBox
from Tkinter import *
from Tkconstants import *
from FileDialog import *  
from io import *
from floflags import *


###########################################################################################
###########################################################################################
##													##
##	FLOTTER - dfg to cnf translation									##
##		classes:											##
##			Flotter										##
##													##
###########################################################################################
###########################################################################################



class Flotter:
##########################################################################################
#	presents main window for cnd to dfg conversion; handles the whole Flotter-Flag-Configuration		#
#		flags:											#
#			CNFOptSkolem									#
#			CNFProofSteps									#
#			CNFStrSkolem									#
#			CNFRenaming									#
#			POptSkolem									#
#			PStrSkolem									#
#			CNFPRenaming									#
#			Flotter										#
#													#
#		methods:										#
#			__init__(self,parent=None,directory) ... initialises flags and window			#
#			load(self) ... loads flag-configuration						#
#			save(self) ... saves configuration							#
#			quit(self) ... quits if dictionary could be made						#
#			cancel(self) ... cancels - dictionary["Cancel"]="T"					#
#			doFlagLib(self) ... makes dictionary of flag-values					#
#			set_Var(self,dictionary) ... sets flag-values according to dictionary			#
#			show(self,shw) ... if shw="SHOW" shows window, at end returns flag-dictionary		#
#						{flagname:flagvalue,...}, flotter:1 (only do flotter)		#
#					else retuns default flag-dictionary, flotter:0 (also do SPASS)		#
##########################################################################################
	def __init__(self,parent=None,directory):
		self.top = Toplevel()
		self.directory=directory	
		self.top.title("DFG to CNF Flags")
#		self.frame=Frame(self.top)
#		self.frame.pack(side=BOTTOM)
		self.menu_frame=Frame(self.top,relief=RAISED,bd=2)
		self.menu_frame.pack(side=TOP,fill=X)
		#	menu				#
		ConvertButton=Menubutton(self.menu_frame,text="Convert")
		ConvertButton.pack(side=LEFT)
		self.ConvertMenu=Menu(ConvertButton)
		self.ConvertMenu.add_command(label="Start",command=self.quit)
		self.ConvertMenu.add_command(label="Cancel",command=self.cancel)
		ConvertButton["menu"]=self.ConvertMenu
		SettingButton=Menubutton(self.menu_frame,text="Setting")
		SettingButton.pack(side=LEFT)
		self.SettingMenu=Menu(SettingButton)
		self.SettingMenu.add_command(label="Load",command=self.load)
		self.SettingMenu.add_command(label="Save",command=self.save)
		SettingButton["menu"]=self.SettingMenu
		#	flags				#		
		self.f_f=flotter_flags(self)
		self.f_f.frame.pack(side=BOTTOM,padx=10)
		#	initialise dictionary		#
		self.dict={}
		self.dict["Error"]="F"
		self.dict["Cancel"]=""
		self.dict["Flotter"]=1

	def load(self):
		if os.path.exists(self.directory):
			fd=LoadFileDialog(self.top)
			filename = fd.go(self.directory,"*.flg")
			if filename<>None:
				x=FlagIO().load_SPASS(filename,8)
				if x<>"ERROR":
					self.set_Var(x,filename)
		else:
			tkMessageBox.showerror("Directory","Directory does not exist - \n" + self.directory)
		
	def save(self):
		n=NewFileName().get(self.directory,"flotter_flag","flg")
		if n<>"ERROR":
			fd=SaveFileDialog(self.top)
			filename = fd.go(self.directory,"*.flg", n)
			if filename<>None:
				self.doFlagLib()
				if self.dict["Error"]=="F":
					FlagIO().save_SPASS(filename,self.dict)
				self.dict["Error"]="F"

	def quit(self):
		self.doFlagLib()
		if self.dict["Error"]=="F":
			self.dict["Cancel"]="F"
			self.top.destroy()
		self.dict["Error"]="F"

	def cancel(self):
		self.dict["Cancel"]="T"
		self.top.destroy()

	def doFlagLib(self):
		self.dict["POptSkolem"]=self.f_f._POptSkolem.get()
		self.dict["PStrSkolem"]=self.f_f._PStrSkolem.get()
		self.dict["CNFPRenaming"]=self.f_f._CNFPRenaming.get()
		self.dict["CNFOptSkolem"]=self.f_f._CNFOptSkolem.get()
		self.dict["CNFStrSkolem"]=self.f_f._CNFStrSkolem.get()
		self.dict["CNFRenaming"]=self.f_f._CNFRenaming.get()
		try:
			self.dict["CNFProofSteps"]=eval(self.f_f._CNFProofSteps.get())
		except:
			tkMessageBox.showerror("Wrong Value","Wrong Value for 'Restriction Of Number Of Optimized Skolemization ProofSteps'")
			self.dict["Error"]="T"

	def set_Var(self,dictionary,filename=""):
		if ((0 <= eval(dictionary["POptSkolem"]) <= 1) 
		and (0 <= eval(dictionary["PStrSkolem"]) <= 1)
		and (0 <= eval(dictionary["CNFPRenaming"]) <= 1)
		and (0 <= eval(dictionary["CNFOptSkolem"]) <= 1)
		and (0 <= eval(dictionary["CNFStrSkolem"]) <= 1)
		and (0 <= eval(dictionary["CNFRenaming"]) <= 1)):
			try:
				self.f_f._CNFOptSkolem.set(eval(dictionary["CNFOptSkolem"]))
				self.f_f._CNFProofSteps.set(abs(eval(dictionary["CNFProofSteps"])))
				self.f_f._CNFStrSkolem.set(eval(dictionary["CNFStrSkolem"]))
				self.f_f._CNFRenaming.set(eval(dictionary["CNFRenaming"]))
				self.f_f._POptSkolem.set(eval(dictionary["POptSkolem"]))
				self.f_f._PStrSkolem.set(eval(dictionary["PStrSkolem"]))
				self.f_f._CNFPRenaming.set(eval(dictionary["CNFPRenaming"]))
			except:
				tkMessageBox.showerror("File","Error in Flag-File - bad flag-definition found - \n" + filename + "\nNot all flags loaded!")
		else:
			tkMessageBox.showerror("File","Error in Flag-File - bad flag-definition found - \n" + filename + "\nNothing loaded!")

	def show(self,shw):
		if shw=="SHOW":
			self.top.deiconify()
			self.top.wait_window()
			if self.dict["Cancel"]=="":				#	window closed through X		#
				tkMessageBox.showinfo("","Cancel")
				self.dict["Cancel"]="T"
		else:
			self.dict["Flotter"]=0
			self.quit()
		return self.dict



