#!/usr/local/bin/python
import sys
import os
import string
import glob
import tkMessageBox
from Tkinter import *
from Tkconstants import *
from FileDialog import LoadFileDialog
from FileDialog import SaveFileDialog
from spagrp import *
from io import *


###########################################################################################
###########################################################################################
##													##
##	WINDOW TO SET SPASS FLAGS									##
##			classes:										##
##				SPASS_Flags								##
##													##
###########################################################################################
###########################################################################################

class SPASS_Flags:
##########################################################################################
#	presents main window to set SPASS flags; handles the whole SPASS-Flag-Configuration			#
#		methods:										#
#			__init__(self,parent=None,main_directory,directory)					#	
#			chc(self) ... changes button-colour and shows flags					#
#			opt(self) ... changes button-colour and shows flags					#
#			pt(self) ... changes button-colour and shows flags					#
#			ifr(self) ... changes button-colour and shows flags					#
#			redr(self) ... changes button-colour and shows flags					#
#			wgt(self) ... changes button-colour and shows flags					#
#			skl(self) ... changes button-colour and shows flags					#
#			cl(self) ... changes button-colour and shows flags					#
#			syst(self) ... changes button-colour and shows flags					#
#			deltn(self) ... changes button-colour and shows flags					#
#			autm(self) ... changes button-colour and shows flags					#
#			mk_b(self) ... changes button-color to black						#
#			hide_all_radios(self) ... sets all flag-groups invisible					#
#			quit(self) ... quits the flag-application if settings are ok					#
#			cancel(self) ... quits the flag-application; dictionary["Cancel"]="T"			#
#			save_set(self) ... saves the flag-configuration to a file					#
#			load(self) ... loads a flag-configuration from a file					#
#			check_Var(self,dictionary,filename) ... checks flagvalues 				#
#			set_Var(self,dictionary,filename) ... sets the flag-variables from dictionary-entrys		#
#			doFlagLib(self) ... returns dictionary of flag-values					#
#			show(self,shw) ... if shw="SHOW" shows window, when destroyed returns flag-dictionary#
#					else returns default flag-dictionary					#
#		uses classes:										#
#			choose_calc(self)									#
#			outpt										#
#			prnt										#
#			inference_rule									#
#			reduction									#
#			weight										#
#			skolemisation									#
#			calculus										#
#			system										#
#			deletion										#
##########################################################################################

	###################################################################################
	#	__init__											#
	###################################################################################
	def __init__(self,parent=None,directory,defaults,last):
		###################################
		#	menu and buttons		#
		###################################
		#	Frames for menu and buttons	#
		self.top = Toplevel()
		self.frame1 = Frame(self.top)
		self.directory=directory
		self.defaults=defaults
		self.last=last
		self.top.title("SPASS-Flags")
		self.menu_frame=Frame(self.top,relief=RAISED,bd=2)
		self.menu_frame.pack(side=TOP,fill=X)
		self.frame1.pack(side=LEFT,padx=10,anchor=N)
		#	buttons				#
		label = Label(self.frame1, text="autonomous mode:")
		label.grid(row=0, column=0,sticky="E")
		self.Auto=IntVar()
		self.Auto.set(0)
		radio1=Radiobutton(self.frame1,text="on",variable=self.Auto,value=1,command=self.auto)
		radio2=Radiobutton(self.frame1,text="off",variable=self.Auto,value=0,command=self.auto)
		radio1.grid(row=0,column=1,sticky="W")
		radio2.grid(row=1,column=1,sticky="W")
		self.cc_button=Button(self.frame1,text="Choose calculus",relief=FLAT,command=self.chc)
		self.cc_button.grid(row=2,column=1,sticky="E")
		self.op_button=Button(self.frame1,text="Output",relief=FLAT,command=self.opt)
		self.op_button.grid(row=3,column=1,sticky="E")
		self.p_button=Button(self.frame1,text="Print",relief=FLAT,command=self.pt)
		self.p_button.grid(row=4,column=1,sticky="E")
		self.ir_button=Button(self.frame1,text="Inference rules",relief=FLAT,command=self.ifr)
		self.ir_button.grid(row=5,column=1,sticky="E")
		self.red_button=Button(self.frame1,text="Reduction rules",relief=FLAT,command=self.redr)
		self.red_button.grid(row=6,column=1,sticky="E")
		self.wt_button=Button(self.frame1,text="Weight",relief=FLAT,command=self.wgt)
		self.wt_button.grid(row=7,column=1,sticky="E")
		self.c_button=Button(self.frame1,text="Calculus",relief=FLAT,command=self.cl)
		self.c_button.grid(row=9,column=1,sticky="E")
		self.sys_button=Button(self.frame1,text="System",relief=FLAT,command=self.syst)
		self.sys_button.grid(row=10,column=1,sticky="E")
		self.delt_button=Button(self.frame1,text="Deletion",relief=FLAT,command=self.deltn)
		self.delt_button.grid(row=11,column=1,sticky="E")
		#	menu				#
		setButton=Menubutton(self.menu_frame,text="Setting")
		setButton.pack(side=LEFT)
		self.setMenu=Menu(setButton)
		self.setMenu.add_command(label="Save",command=self.save_set)
		self.setMenu.add_command(label="Load",command=self.load)
		self.setMenu.add_separator()
		self.setMenu.add_command(label="Load Standard",command=self.load_standard)
		self.setMenu.add_separator()
		self.setMenu.add_command(label="Cancel",command=self.cancel)
		self.setMenu.add_command(label="Accept",command=self.quit)
		setButton["menu"]=self.setMenu
		###################################
		#	initialises classes for flag-groups	#
		#	initialises dictionary		#
		#	initialises directory		#
		###################################
		self.cc=choose_calc(self)
		self.p=prnt(self)
		self.ir=inference_rule(self)
		self.red=reduction(self)
		self.wt=weight(self)
		self.c=calculus(self)
		self.sys=system(self)
		self.delt=deletion(self)
		self.op=outpt(self)
		self.dict={}
		self.dict["Cancel"]=""
		self.dict["Error"]="F"
		self.dict["Auto"]="0"

	###################################################################################
	#	button-functions										#
	###################################################################################

	def chc(self):
		self.mk_b()
		self.cc_button.config(foreground="red")
		self.hide_all_radios()
		self.cc.frame.pack(side=RIGHT,padx=10)

	def opt(self):
		self.mk_b()
		self.op_button.config(foreground="red")
		self.hide_all_radios()
		self.op.frame.pack(side=RIGHT,padx=10)
		self.op.check_RFSub()
		self.op.check_RSST()
		self.op.check_RDST()
		self.op.check_RFRBRew()
		self.op.check_RCon()
		self.op.check_RTaut()
		self.op.check_RObv()
		self.op.check_RSSi()
		self.op.check_RUnC()
		self.op.check_RFRBClR()

	def pt(self):
		self.mk_b()
		self.p_button.config(foreground="red")
		self.hide_all_radios()
		self.p.frame.pack(side=RIGHT,padx=10)

	def ifr(self):
		self.mk_b()
		self.ir_button.config(foreground="red")
		self.hide_all_radios()
		self.ir.frame.pack(side=RIGHT,padx=10)

	def redr(self):
		self.mk_b()
		self.red_button.config(foreground="red")
		self.hide_all_radios()
		self.red.frame.pack(side=RIGHT,padx=10)

	def wgt(self):
		self.mk_b()
		self.wt_button.config(foreground="red")
		self.hide_all_radios()
		self.wt.frame.pack(side=RIGHT,padx=10)

	def cl(self):
		self.mk_b()
		self.c_button.config(foreground="red")
		self.hide_all_radios()
		self.c.frame.pack(side=RIGHT,padx=10)
		self.c.warn_Select()
		self.c.warn_Sorts()

	def syst(self):
		self.mk_b()
		self.sys_button.config(foreground="red")
		self.hide_all_radios()
		self.sys.frame.pack(side=RIGHT,padx=10)

	def deltn(self):
		self.mk_b()
		self.delt_button.config(foreground="red")
		self.hide_all_radios()
		self.delt.frame.pack(side=RIGHT,padx=10)


	def mk_b(self):
		self.cc_button.config(foreground="black")
		self.op_button.config(foreground="black")
		self.p_button.config(foreground="black")
		self.ir_button.config(foreground="black")
		self.red_button.config(foreground="black")
		self.wt_button.config(foreground="black")
		self.c_button.config(foreground="black")
		self.sys_button.config(foreground="black")
		self.delt_button.config(foreground="black")

	def  hide_all_radios(self):
		self.cc.frame.forget()
		self.op.frame.forget()
		self.p.frame.forget()
		self.ir.frame.forget()
		self.red.frame.forget()
		self.wt.frame.forget()
		self.c.frame.forget()
		self.sys.frame.forget()
		self.delt.frame.forget()

	def auto(self):
		if self.Auto.get()==1:
			self.mk_b()
			self.hide_all_radios()
			self.cc_button["state"]=DISABLED
			self.op_button["state"]=DISABLED
			self.p_button["state"]=DISABLED
			self.ir_button["state"]=DISABLED
			self.red_button["state"]=DISABLED
			self.wt_button["state"]=DISABLED
			self.c_button["state"]=DISABLED
			self.sys_button["state"]=DISABLED
			self.delt_button["state"]=DISABLED
			self.setMenu.entryconfig("Save",state=DISABLED)
			self.dict["Auto"]="1"
			tkMessageBox.showinfo("Modelbuilding","In automatic mode correct modelbuilding can't be assured because of calculi 'Sorts' and 'Select'")
		else:
			self.cc_button["state"]=NORMAL
			self.op_button["state"]=NORMAL
			self.p_button["state"]=NORMAL
			self.ir_button["state"]=NORMAL
			self.red_button["state"]=NORMAL
			self.wt_button["state"]=NORMAL
			self.c_button["state"]=NORMAL
			self.sys_button["state"]=NORMAL
			self.delt_button["state"]=NORMAL
			self.setMenu.entryconfig("Save",state=NORMAL)
			self.dict["Auto"]="0"

	

	###################################################################################
	#	menu-functions										#
	###################################################################################

	def quit(self):
		self.doFlagLib()
		if self.dict["Error"]=="F":
			self.dict["Cancel"]="F"
			self.top.destroy()
		self.dict["Error"]="F"

	def cancel(self):
		self.dict["Cancel"]="T"
		self.top.destroy()

	def save_set(self):
		n=NewFileName().get(self.directory,"spass_flag","flg")
		if n<>"ERROR":
			fd=SaveFileDialog(self.top)
			filename = fd.go(self.directory,"*.flg", n)
			if filename<>None:
				self.doFlagLib()
				if self.dict["Error"]=="F":
					FlagIO().save_SPASS(filename,self.dict)
				self.dict["Error"]="F"

	def load_standard(self):
		self.set_Var(self.defaults,"")

	def load(self):
		if os.path.exists(self.directory):
			fd=LoadFileDialog(self.top)
			filename = fd.go(self.directory,"*.flg")
			if filename<>None:
				x=FlagIO().load_SPASS(filename,71)
				self.set_Var(x,filename)
		else:
			tkMessageBox.showerror("Directory","Directory does not exist - \n" + self.directory)



	###################################################################################
	#	check flag-settings									#
	###################################################################################
	def check_Var(self,x,filename):
		try:
			if ((0 <= x["CPara"] <= 1)
			and (0 <= x["COrdPara"] <= 1)
			and (0 <= x["CSupE"] <= 1)
			and (0 <= x["CSupP"] <= 1)
			and (0 <= x["Sorts"] <= 2)
			and (0 <= x["Select"] <=2)
			and (0 <= x["RSST"] <= 1)
			and (0 <= x["RDST"] <= 1)
			and (0 <= x["SatInput"] <= 1)
			and (0 <= x["PSub"] <= 1)
			and (0 <= x["DocSST"] <= 1)
			and (0 <= x["DocDST"] <= 1)
			and (0 <= x["PRew"] <= 1)
			and (0 <= x["PCon"] <= 1)
			and (0 <= x["PTaut"] <= 1)
			and (0 <= x["PObv"] <= 1)
			and (0 <= x["PSSi"] <= 1)
			and (0 <= x["PSST"] <= 1)
			and (0 <= x["DocProof"] <= 1)
			and (0 <= x["PClR"] <= 1)
			and (0 <= x["PDer"] <= 1)
			and (0 <= x["PGiven"] <= 1)
			and (0 <= x["PKept"] <= 1)
			and (0 <= x["DocSplit"] <= 1)
			and (0 <= x["PProblem"] <= 1)
			and (0 <= x["PEmptyClause"] <= 1)
			and (0 <= x["PStatistic"] <= 1)
			and (0 <= x["PFlags"] <= 1)
			and (0 <= x["DocFormulaeOnly"] <= 1)
			and (0 <= x["PUnC"] <= 1)
			and (0 <= x["PApplyDefs"] <= 1)
			and (0 <= x["FPModel"] <= 1)
			and (0 <= x["FPDFGProof"] <= 1)
			and (0 <= x["IEmS"] <= 1)
			and (0 <= x["ISoR"] <= 1)
			and (0 <= x["IEqR"] <= 1)
			and (0 <= x["IEqF"] <= 1)
			and (0 <= x["IMPm"] <= 1)
			and (0 <= x["ISpR"] <= 1)
			and (0 <= x["ISpL"] <= 1)
			and (0 <= x["IGeR"] <= 1)
			and (0 <= x["IGeF"] <= 1)
			and (0 <= x["IBUR"] <= 1)
			and (0 <= x["IUnR"] <= 1)
			and (0 <= x["IOPm"] <= 1)
			and (0 <= x["ISPm"] <= 1)
			and (0 <= x["ISHy"] <= 1)
			and (0 <= x["IOHy"] <= 1)
			and (0 <= x["IDEF"] <= 1)
			and (0 <= x["RFRew"] <= 1)
			and (0 <= x["RBRew"] <= 1)
			and (0 <= x["RFSub"] <= 1)
			and (0 <= x["RBSub"] <= 1)
			and (0 <= x["RCon"] <= 1)
			and (0 <= x["RFClR"] <= 1)
			and (0 <= x["RBClR"] <= 1)
			and (0 <= x["RObv"] <= 1)
			and (0 <= x["RTaut"] <= 1)
			and (0 <= x["RSSi"] <= 1)
			and (0 <= x["RInput"] <= 1)
			and (0 <= x["FullRed"] <= 1)
			and (0 <= x["RUnC"] <= 1)
			and (0 <= x["PrefVar"] <= 1)):
				try:
					x["Loops"]
					x["Splits"]
					x["Memory"]
					x["TimeLimit"]
					return "OK"
				except:
					tkMessageBox.showerror("File","Error 1in Flag-File - bad flag-definition found - \n" + filename + "\nNothing loaded!")
					return "ERROR"
			else:
				tkMessageBox.showerror("File","Error 2in Flag-File - bad flag-definition found - \n" + filename + "\nNothing loaded!")
				return "ERROR"
		except:
			tkMessageBox.showerror("File","Error 3in Flag-File - flag-definition missing - \n" + filename + "\nNothing loaded!")
			return "ERROR"


	###################################################################################
	#	convert to integer										#
	###################################################################################
	def make_Var(self,x):
		try:
			x0={}
			for i in x.values():
				x0[i]=eval(x[i])	
			return x0
		except:
			return x


	###################################################################################
	#	dictionary to flag-settings									#
	###################################################################################
	def set_Var(self,x,filename):
		x=self.make_Var(x)
		if self.check_Var(x,filename)=="OK":
			self.Auto.set(x["Auto"])
			self.auto()
			self.cc._CPara.set(x["CPara"])
			self.cc._COrdPara.set(x["COrdPara"])
			self.cc._CSupE.set(x["CSupE"])
			self.cc._CSupP.set(x["CSupP"])
			if x["Loops"]==-1:
				self.c._Loops.set(1)
				self.c.loopnum.set('')
			else:
				self.c._Loops.set(0)
				self.c.loopnum.set(str(abs(x["Loops"])))
			if x["Splits"]==-1:
				self.c._Splits.set(1)
				self.c.splitnum.set('')
			else:
				self.c._Splits.set(0)
				self.c.splitnum.set(str(abs(x["Splits"])))
			self.c._Sorts.set(x["Sorts"])
			self.c._Select.set(x["Select"])
			self.c._RSST.set(x["RSST"])
			self.c._RDST.set(x["RDST"])
			self.c._SatInput.set(x["SatInput"])
			self.op._PSub.set(x["PSub"])
			self.op._DocSST.set(x["DocSST"])
			self.op._DocDST.set(x["DocDST"])
			self.op._PRew.set(x["PRew"])
			self.op._PCon.set(x["PCon"])
			self.op._PTaut.set(x["PTaut"])
			self.op._PObv.set(x["PObv"])
			self.op._PSSi.set(x["PSSi"])
			self.op._PSST.set(x["PSST"])
			self.op._DocProof.set(x["DocProof"])
			self.op._PClR.set(x["PClR"])
			self.op._PDer.set(x["PDer"])
			self.op._PGiven.set(x["PGiven"])
			self.op._PKept.set(x["PKept"])
			self.op._DocSplit.set(x["DocSplit"])
			self.op._PProblem.set(x["PProblem"])
			self.op._PEmptyClause.set(x["PEmptyClause"])
			self.op._PStatistic.set(x["PStatistic"])
			self.op._PFlags.set(x["PFlags"])
			self.op._DocFormulaeOnly.set(x["DocFormulaeOnly"])
			self.op._PUnC.set(x["PUnC"])
			self.op._PApplyDefs.set(x["PApplyDefs"])
			self.p._FPModel.set(x["FPModel"])
			self.p._FPDFGProof.set(x["FPDFGProof"])
			if x["Memory"]==-1:
				self.sys._Memory.set(1)
				self.sys.memnum.set('')
			else:
				self.sys._Memory.set(0)
				self.sys.memnum.set(str(abs(x["Memory"])))
			if x["TimeLimit"]==-1:
				self.sys._TimeLimit.set(1)
				self.sys.secnum.set('')
			else:
				self.sys._TimeLimit.set(0)
				self.sys.secnum.set(str(abs(x["TimeLimit"])))
			self.ir._IEmS.set(x["IEmS"])
			self.ir._ISoR.set(x["ISoR"])
			self.ir._IEqR.set(x["IEqR"])
			self.ir._IEqF.set(x["IEqF"])
			self.ir._IMPm.set(x["IMPm"])
			self.ir._ISpR.set(x["ISpR"])
			self.ir._ISpL.set(x["ISpL"])
			self.ir._IGeR.set(x["IGeR"])
			self.ir._IGeF.set(x["IGeF"])
			self.ir._IBUR.set(x["IBUR"])
			self.ir._IUnR.set(x["IUnR"])
			self.ir._IOPm.set(x["IOPm"])
			self.ir._ISPm.set(x["ISPm"])
			self.ir._ISHy.set(x["ISHy"])
			self.ir._IOHy.set(x["IOHy"])
			self.ir._IDEF.set(x["IDEF"])
			self.delt._RFRew.set(x["RFRew"])
			self.delt._RBRew.set(x["RBRew"])
			self.delt._RFSub.set(x["RFSub"])
			self.delt._RBSub.set(x["RBSub"])
			self.delt._RCon.set(x["RCon"])
			self.red._RFClR.set(x["RFClR"])
			self.red._RBClR.set(x["RBClR"])
			self.red._RObv.set(x["RObv"])
			self.red._RTaut.set(x["RTaut"])
			self.red._RSSi.set(x["RSSi"])
			self.red._RInput.set(x["RInput"])
			self.red._FullRed.set(x["FullRed"])
			self.red._RUnC.set(x["RUnC"])
			self.wt._WDRatio.set(str(abs(x["WDRatio"])))
			self.wt._FuncWeight.set(str(abs(x["FuncWeight"])))
			self.wt._VarWeight.set(str(abs(x["VarWeight"])))
			self.wt._PrefVar.set(x["PrefVar"])

	###################################################################################
	#	flag-settings to dictionary 									#
	###################################################################################

	def doFlagLib(self):
		for i in [1]:
			self.dict["CPara"]=self.cc._CPara.get()
			self.dict["COrdPara"]=self.cc._COrdPara.get()
			self.dict["CSupE"]=self.cc._CSupE.get()
			self.dict["CSupP"]=self.cc._CSupP.get()
			if self.c._Loops.get()==1:
				self.dict["Loops"]=-1
			else:
				try:
					self.dict["Loops"]=eval(self.c.loopnum.get())
				except:
					tkMessageBox.showwarning("Wrong Value","Wrong Value for 'Number of Loops'")
					self.dict["Error"]="T"
					break
			if self.c._Splits.get()==1:
				self.dict["Splits"]=-1
			else:
				try:
					self.dict["Splits"]=eval(self.c.splitnum.get())
				except:
					tkMessageBox.showwarning("Wrong Value","Wrong Value for 'Number of Splits")
					self.dict["Error"]="T"
					break
			self.dict["Sorts"]=self.c._Sorts.get()
			self.dict["RSST"]=self.c._RSST.get()
			self.dict["Select"]=self.c._Select.get()
			self.dict["RDST"]=self.c._RDST.get()
			self.dict["SatInput"]=self.c._SatInput.get()
			self.dict["PSub"]=self.op._PSub.get()
			self.dict["DocSST"]=self.op._DocSST.get()
			self.dict["DocDST"]=self.op._DocDST.get()
			self.dict["PRew"]=self.op._PRew.get()
			self.dict["PCon"]=self.op._PCon.get()
			self.dict["PTaut"]=self.op._PTaut.get()
			self.dict["PObv"]=self.op._PObv.get()
			self.dict["PSSi"]=self.op._PSSi.get()
			self.dict["PSST"]=self.op._PSST.get()
			self.dict["DocProof"]=self.op._DocProof.get()
			self.dict["PClR"]=self.op._PClR.get()
			self.dict["PDer"]=self.op._PDer.get()
			self.dict["PGiven"]=self.op._PGiven.get()
			self.dict["PKept"]=self.op._PKept.get()
			self.dict["DocSplit"]=self.op._DocSplit.get()
			self.dict["PProblem"]=self.op._PProblem.get()
			self.dict["PEmptyClause"]=self.op._PEmptyClause.get()
			self.dict["PStatistic"]=self.op._PStatistic.get()
			self.dict["PFlags"]=self.op._PFlags.get()
			self.dict["DocFormulaeOnly"]=self.op._DocFormulaeOnly.get()
			self.dict["PUnC"]=self.op._PUnC.get()
			self.dict["PApplyDefs"]=self.op._PApplyDefs.get()
			self.dict["FPModel"]=self.p._FPModel.get()
			self.dict["FPDFGProof"]=self.p._FPDFGProof.get()
			if self.sys._Memory.get()==1:
				self.dict["Memory"]=-1
			else:
				try:
					self.dict["Memory"]=eval(self.sys.memnum.get())
				except:
					tkMessageBox.showwarning("Wrong Value","Wrong Value for 'Maximal Memory Allocation'")
					self.dict["Error"]="T"
					break
			if self.sys._TimeLimit.get()==1:
				self.dict["TimeLimit"]=-1
			else:
				try:
					self.dict["TimeLimit"]=eval(self.sys.secnum.get())
				except:
					tkMessageBox.showwarning("Wrong Value","Wrong Value for 'amount of sec available to SPASS")
					self.dict["Error"]="T"
					break
			self.dict["IEmS"]=self.ir._IEmS.get()
			self.dict["ISoR"]=self.ir._ISoR.get()
			self.dict["IEqR"]=self.ir._IEqR.get()
			self.dict["IEqF"]=self.ir._IEqF.get()
			self.dict["IMPm"]=self.ir._IMPm.get()
			self.dict["ISpR"]=self.ir._ISpR.get()
			self.dict["ISpL"]=self.ir._ISpL.get()
			self.dict["IGeR"]=self.ir._IGeR.get()
			self.dict["IGeF"]=self.ir._IGeF.get()
			self.dict["IBUR"]=self.ir._IBUR.get()
			self.dict["IUnR"]=self.ir._IUnR.get()
			self.dict["IOPm"]=self.ir._IOPm.get()
			self.dict["ISPm"]=self.ir._ISPm.get()
			self.dict["ISHy"]=self.ir._ISHy.get()
			self.dict["IOHy"]=self.ir._IOHy.get()
			self.dict["IDEF"]=self.ir._IDEF.get()
			self.dict["RFRew"]=self.delt._RFRew.get()
			self.dict["RBRew"]=self.delt._RBRew.get()
			self.dict["RFSub"]=self.delt._RFSub.get()
			self.dict["RBSub"]=self.delt._RBSub.get()
			self.dict["RCon"]=self.delt._RCon.get()
			self.dict["RFClR"]=self.red._RFClR.get()
			self.dict["RBClR"]=self.red._RBClR.get()
			self.dict["RObv"]=self.red._RObv.get()
			self.dict["RTaut"]=self.red._RTaut.get()
			self.dict["RSSi"]=self.red._RSSi.get()
			self.dict["RUnC"]=self.red._RUnC.get()
			self.dict["RInput"]=self.red._RInput.get()
			self.dict["FullRed"]=self.red._FullRed.get()
			try:
				self.dict["WDRatio"]=eval(self.wt._WDRatio.get())
			except:
				tkMessageBox.showwarning("Wrong Value","Wrong Value for 'Ratio'")
				self.dict["Error"]="T"
				break
			try:
				self.dict["FuncWeight"]=eval(self.wt._FuncWeight.get())
			except:
				tkMessageBox.showwarning("Wrong Value","Wrong Value for 'Weigth Of A Function Symbol'")
				self.dict["Error"]="T"
				break
			try:
				self.dict["VarWeight"]=eval(self.wt._VarWeight.get())
			except:
				tkMessageBox.showwarning("Wrong Value","Wrong Value for 'Weigth Of A Variable Symbol'")
				self.dict["Error"]="T"
				break
			self.dict["PrefVar"]=self.wt._PrefVar.get()

	###################################################################################
	#	show window - when distroyed destroy flag-group-objects and return dictionary of flag-settings	#
	###################################################################################
	
	def show(self,shw=""):
		if shw=="SHOW":
			self.top.deiconify()
			self.set_Var(self.last,"")
			self.top.wait_window()
			if self.dict["Cancel"]=="":
				tkMessageBox.showinfo("","Cancel")
				self.dict["Cancel"]="T"
		else:
			self.quit()
		return self.dict

