#!/usr/local/bin/python
import sys
import os
import string
import glob
import tkMessageBox
from Tkinter import *
from Tkconstants import *
from FileDialog import *  
from io import *
from tptp2x_h import *
from showout import *


###########################################################################################
###########################################################################################
##													##
##	TPTP2X - 											##
##		classes:											##
##			TPTP2X										##
##													##
###########################################################################################
###########################################################################################



class TPTP2X:
##########################################################################################
#	presents main window for cnd to dfg conversion; handles the whole Flotter-Flag-Configuration		#
#		flags:											#
#			CNFOptSkolem									#
#			CNFProofSteps									#
#			CNFStrSkolem									#
#			CNFRenaming									#
#			POptSkolem									#
#			PStrSkolem									#
#			CNFPRenaming									#
#													#
#		methods:										#
#			__init__(self,parent=None) ... initialises flags and window				#
#			load(self) ... loads flag-configuration						#
#			save(self) ... saves configuration							#
#			quit(self) ... quits the application							#
#			doFlagLib(self) ... makes dictionary of flag-values					#
#			set_Var(self,dictionary) ... sets flag-values according to dictionary			#
#			do_convert(self) ... writes flags to dfg file and calls Flotter				#
#			show(self) ... shows window							#
##########################################################################################
	def __init__(self,parent=None,fkt="",directory):
		self.top = Toplevel()
		t="TPTP 2 " + fkt
		self.directory=directory
		self.fkt=fkt
		self.top.title(t)
		self.menu_frame=Frame(self.top,relief=RAISED,bd=2)
		self.menu_frame.pack(side=TOP,fill=X)
		self.frame1=Frame(self.top)
		self.frame1.pack(side=LEFT,anchor=N, padx=5)
		

		self.q_button=Button(self.frame1,text="Quietness",relief=FLAT,command=self.q)
		self.q_button.grid(row=0,column=1,sticky="E")
		self.u_button=Button(self.frame1,text="Usage",relief=FLAT,command=self.u)
		self.u_button.grid(row=1,column=1,sticky="E")
		self.t_button=Button(self.frame1,text="Transform",relief=FLAT,command=self.t)
		self.t_button.grid(row=2,column=1,sticky="E")

		self.convert_start=Button(self.frame1,text="Start",command=self.do_convert)
		self.convert_start.grid(row=3,column=1,sticky="WE")
		self.convert_cancel=Button(self.frame1,text="Cancel",command=self.quit)
		self.convert_cancel.grid(row=4,column=1,sticky="WE")
		self.convert_load=Button(self.frame1,text="Load",command=self.do_convert)
		self.convert_load.grid(row=5,column=1,sticky="WE")
		self.convert_save=Button(self.frame1,text="Save",command=self.quit)
		self.convert_save.grid(row=6,column=1,sticky="WE")

		self.loq=level_of_quietness(self)
		self.ui=usage_info(self)
		self.tf=transform(self,self.fkt)

		self.dict={}
		self.tptp=""

	def q(self):
		self.mk_b()
		self.q_button.config(foreground="red")
		self.hide_all_radios()
		self.loq.frame.pack(padx=10,side=RIGHT, anchor=N)

	def u(self):
		self.mk_b()
		self.u_button.config(foreground="red")
		self.hide_all_radios()
		self.ui.frame.pack(padx=10,side=RIGHT,anchor=N)

	def t(self):
		self.mk_b()
		self.t_button.config(foreground="red")
		self.hide_all_radios()
		self.tf.frame.pack(padx=10,side=RIGHT,anchor=N)

	def mk_b(self):
		self.q_button.config(foreground="black")
		self.u_button.config(foreground="black")
		self.t_button.config(foreground="black")

	def  hide_all_radios(self):
		self.loq.frame.forget()
		self.ui.frame.forget()
		self.tf.frame.forget()

	def load(self):
		if os.path.exists(self.directory):
			fd=LoadFileDialog(self.top)
			filename = fd.go(self.directory,"*.flg")
			if filename<>None:
				x=FlagIO().load_TPTP(filename,21)
				if x<>"ERROR":
					self.set_Var(x,filename)
		else:
			tkMessageBox.showerror("Directory","Directory does not exist - \n" + self.directory)
		
	def save(self):
		n=NewFileName().get(self.directory,"TPTP_flag","flg")
		if n<>"ERROR":
			fd=SaveFileDialog(self.top)
			filename = fd.go(self.directory,"*.flg", n)
			if filename<>None:
				self.doFlagLib()
				FlagIO().save_TPTP(filename,self.dict)

	def quit(self):
		self.top.destroy()
		self.tptp="CANCEL"

	def doFlagLib(self):
		self.dict["q"]=self.loq.q.get()
		self.dict["h"]=self.ui.h.get()
		self.dict["t"]=self.tf.t.get()
		self.dict["clausify"]=self.tf.clausify.get()
		self.dict["simplify"]=self.tf.simplify.get()
		self.dict["cnf"]=self.tf.cnf.get()
		self.dict["lr"]=self.tf.lr.get()
		self.dict["cr"]=self.tf.cr.get()
		self.dict["clr"]=self.tf.clr.get()
		self.dict["fr"]=self.tf.fr.get()
		self.dict["random"]=self.tf.random.get()
		self.dict["er"]=self.tf.er.get()
		self.dict["ran_er"]=self.tf.ran_er.get()
		self.dict["rm_equality"]=self.tf.rm_equality.get()
		self.dict["add_equality"]=self.tf.add_equality.get()
		self.dict["rf"]=self.tf.req_dict["f"]
		self.dict["rt"]=self.tf.req_dict["t"]
		self.dict["rr"]=self.tf.req_dict["r"]
		self.dict["rs"]=self.tf.req_dict["s"]
		self.dict["rp"]=self.tf.req_dict["p"]
		self.dict["cl_val"]=self.tf.cl_val

	def set_Var(self,dictionary,filename=""):
		try:
			self.loq.q.set(eval(dictionary["q"]))
			self.ui.h.set(eval(dictionary["h"]))
			self.tf.t.set(eval(dictionary["t"]))
			self.tf.clausify.set(eval(dictionary["clausify"]))
			self.tf.simplify.set(eval(dictionary["simplify"]))
			self.tf.cnf.set(eval(dictionary["cnf"]))
			self.tf.lr.set(eval(dictionary["lr"]))
			self.tf.cr.set(eval(dictionary["cr"]))
			self.tf.clr.set(eval(dictionary["clr"]))
			self.tf.fr.set(eval(dictionary["fr"]))
			self.tf.random.set(eval(dictionary["random"]))
			self.tf.er.set(eval(dictionary["er"]))
			self.tf.ran_er.set(eval(dictionary["ran_er"]))
			self.tf.rm_equality.set(eval(dictionary["rm_equality"]))
			self.tf.add_equality.set(eval(dictionary["add_equality"]))
			self.tf.req_dict["f"]=eval(dictionary["rf"])
			self.tf.req_dict["t"]=eval(dictionary["rt"])
			self.tf.req_dict["r"]=eval(dictionary["rr"])
			self.tf.req_dict["s"]=eval(dictionary["rs"])
			self.tf.req_dict["p"]=eval(dictionary["rp"])
			self.tf.cl_val=eval(dictionary["cl_val"])
			
		except:
			tkMessageBox.showerror("File","Error in Flag-File - bad flag-definition found - \n" + filename + "\nNot all flags loaded!")

	def do_convert(self):
		t="TPTP 2 " + self.fkt + " Output"
		self.tptp="tptp2X"
		if self.ui.h.get()==1:
			self.tptp=self.tptp + " -h"
		if self.loq.q.get()==3:
			pass
		else:
			self.tptp=self.tptp + " -q" + str(self.loq.q.get())
		if self.tf.t.get()==1:
			t0=self.tptp
			self.tptp=self.tptp + " -t"
			if self.tf.clausify.get()==1:
				self.tptp=self.tptp + "clausify:"
				if self.tf.cl_val==0:
					self.tptp=self.tptp + "bundy,"
				elif self.tf.cl_val==1:
					self.tptp=self.tptp + "otter,"
				elif self.tf.cl_val==2:
					self.tptp=self.tptp + "quaife,"
				else:
					self.tptp=self.tptp + "tptp,"
			if self.tf.simplify.get()==1:
				self.tptp=self.tptp + "simlify,"
			if self.tf.cnf.get()==1:
				self.tptp=self.tptp + "cnf:"
				if self.tf.cl_val==0:
					self.tptp=self.tptp + "bundy,"
				elif self.tf.cl_val==1:
					self.tptp=self.tptp + "otter,"
				elif self.tf.cl_val==2:
					self.tptp=self.tptp + "quaife,"
				else:
					self.tptp=self.tptp + "tptp,"
			if self.tf.lr.get()==1:
				self.tptp=self.tptp + "lr,"
			if self.tf.cr.get()==1:
				self.tptp=self.tptp + "cr,"
			if self.tf.clr.get()==1:
				self.tptp=self.tptp + "clr,"
			if self.tf.fr.get()==1:
				self.tptp=self.tptp + "fr,"
			if self.tf.random.get()==1:
				self.tptp=self.tptp + "random,"
			if self.tf.er.get()==1:
				self.tptp=self.tptp + "er,"
			if self.tf.ran_er.get()==1:
				self.tptp=self.tptp + "ran_er,"
			if self.tf.rm_equality.get()==1:
				self.tptp=self.tptp + "rm_equality:"
				if self.tf.req_dict["f"]==1:
					self.tptp=self.tptp + "f"
				if self.tf.req_dict["t"]==1:
					self.tptp=self.tptp + "t"
				if self.tf.req_dict["r"]==1:
					self.tptp=self.tptp + "r"
				if self.tf.req_dict["s"]==1:
					self.tptp=self.tptp + "s"
				if self.tf.req_dict["p"]==1:
					self.tptp=self.tptp + "p"
				self.tptp=self.tptp+","
			if self.tf.add_equality.get()==1:
				self.tptp=self.tptp + "add_equality,"
			e=len(self.tptp)
			if self.tptp[e-1]==",":
				self.tptp=self.tptp[0:e-1]
			if self.tptp==t0 + " -t":
				tkMessageBox.showinfo("","No Transformations specified - using none")
				self.tptp=t0
		self.top.destroy()

	def show(self):
		self.top.deiconify()
		self.top.wait_window()
		if self.tptp=="":
			tkMessageBox.showinfo("","Cancel")
			self.tptp="CANCEL"
		return self.tptp



