
% mi_pdb.pl
%
%   smallestRulePair/4.

:- module(mi_pdb,[smallestRulePair/6,
	redundantCp/3,
	redundantRule/3,
	redundantEquation/3,
	redundant/2,
	renaming_sv/2,
	renaming_svReflexiv/2]).

:- assertz(user:file_search_path(basic,'.')).
:- assertz(user:file_search_path(trs,'.')).

:- use_module(basic(term)).
:- use_module(basic(queue)).
:- use_module(trs(mi_rep)).
:- use_module(trs(mi_datastr)).
:- use_module(trs(mi_var)).
:- use_module(trs(mi_sort)).
	
% smallestRulePair(+RQueue, +ClosedTree, -TopRule, -RewRule) <--
%    	TopRule & RewRule form the smallest pair of rules in RQueue
%    	uses intern term compare to find smallest rule

smallestRulePair_enumerate(RQueue,TopRule,TRuleNr,RewRule,RRuleNr) :-
    queueMember(TopRule-TRuleNr,RQueue),
    queueMember(RewRule-RRuleNr,RQueue).

smallestRulePair(RQueue0,ClosedTree,TopRule,RewRule,TRuleNr,RRuleNr) :-
    quicksortAxiomLhs(sizeOrd,RQueue0,RQueue),
    smallestRulePair_enumerate(RQueue,TopRule,TRuleNr,RewRule,RRuleNr),
    getValue(TRuleNr,ClosedTree,VsRules),
    \+ memberDs(RRuleNr,VsRules),
    addClosedPair(TRuleNr,RRuleNr,ClosedTree).

:- mapListTree([1,2,3],CTree),getValue(1,CTree,Rs),smallestRulePair(queue(3,[(x*1=>x)-1,(1*x=>x)-2,(i(x)*(x*y)=>y)-3|Xs],Xs),CTree,Rule1,Rule2,R1Nr,R2Nr).

% redundantName(S,T,Queue) <--
% 	Name specifies which type of formel S and T form (equation,rule,critical pair)
% 	S and T are therefor redundant when are (as a formel) a member of Queue

redundantCp(S,T,Queue) :-
	queueMember((cp(S1,T1))-_Index,Queue),
	renaming_svReflexiv(cp(S1,T1),cp(S,T)).

redundantRule(S,T,Queue) :-
	queueMember((S1=>T1)-_Index,Queue),
	renaming_svReflexiv(S1=>T1,S=>T).

redundantEquation(S,T,Queue) :-
	queueMember((S1=T1)-_Index,Queue),
	renaming_svReflexiv(S1=T1,S=T).

redundant(Formel,Queue) :-
	formel(Formel,S,T),
	redundantCp(S,T,Queue).
redundant(Formel,Queue) :-
	formel(Formel,S,T),
	redundantRule(S,T,Queue).
redundant(Formel,Queue) :-
	formel(Formel,S,T),
	redundantEquation(S,T,Queue).

% renaming_sv(+X, +Y, ?Var_dicts) <--	
%	Y is a renaming_sv of  X
%	i.e. X has the same structure, but other
%	variables than Y
%	Var_dicts is a variable list, which
%	contains the necessary assignments to
%	provide a correct renaming_sv
%
%	the postfix "sv" means special variables and is a remark for the
%	exploitation of special user variables

renaming_sv(Term1, Term2) :-
	renaming_sv(Term1, Term2, [],_Pairdicts).

renaming_sv(S, T, VarDicts0, VarDicts) :-
	is_user_var(S), 
	is_user_var(T),
	lookup_atom_atom(S,T,VarDicts0,VarDicts).
renaming_sv(S, T, VarDicts0, VarDicts) :-
	var(S), 
	var(T),
	lookup_atom_atom(S,T,VarDicts0,VarDicts).
renaming_sv(S, T, VarDicts0,VarDicts) :-
        nonvar(S),
	non_user_var(S),
        nonvar(T),
        non_user_var(T),
	functor(S,F,A),
	functor(T,F,A),
	renaming_sv(A,S,T,VarDicts0,VarDicts).

renaming_sv(0,_S,_T,_VarDicts0,_VarDicts).
renaming_sv(N0, S, T, VarDicts0,VarDicts) :-
	N0 > 0,
	arg(N0,S,SArg),
	arg(N0,T,TArg),
	renaming_sv(SArg,TArg,VarDicts0,VarDicts1),
	N is N0 - 1,
	renaming_sv(N,S,T,VarDicts1,VarDicts).

% renaming_svReflexiv(Formel1,Formel2) <--
% 	is the same as renaming_sv/2 but the LHS and RHS of Formel1 and Formel2 
%	respectivelymay be swapt

renaming_svReflexiv(Formel1,Formel2) :-
	formel(Formel1,S1,T1),
	formel(Formel2,S,T),
	renaming_sv((S1,T1),(S,T)).
renaming_svReflexiv(Formel1,Formel2) :-
	formel(Formel1,S1,T1),
	formel(Formel2,S,T),
	renaming_sv((T1,S1),(S,T)).

:- renaming_sv(1*y=y,1*x=x).
:- \+ renaming_svReflexiv(x=a,x=b).
:- renaming_sv(1*X=X,1*Y=Y).
:- renaming_sv(f(X,Y,Z),f(Y,Z,X)).
:- \+ renaming_sv(f(X,Y,Z),f(Y,Z,Y)).
:- formel(a=b,X,Y).
:- redundantEquation(1*u,u,queue(2,[(1*x=x)-1,(i(x)*(x*y)=y)-2|Xs],Xs)).
:- redundantEquation(1*U,U,queue(2,[(1*X=X)-1,(i(X)*(X*Y)=Y)-2|Xs],Xs)).
:- redundant(1*u=u,queue(3,[(1*x=>x)-1,(i(x)*(x*y)=>y)-2,(x*1=>x)-3|Xs],Xs)).
